#!/usr/bin/env python
"""Provides conversion functions to be used during RDFProtoStruct migration."""

from grr_response_core.lib.rdfvalues import client_network as rdf_client_network
from grr_response_proto import jobs_pb2
from grr_response_proto import sysinfo_pb2


def ToProtoNetworkEndpoint(
    rdf: rdf_client_network.NetworkEndpoint,
) -> sysinfo_pb2.NetworkEndpoint:
  return rdf.AsPrimitiveProto()


def ToRDFNetworkEndpoint(
    proto: sysinfo_pb2.NetworkEndpoint,
) -> rdf_client_network.NetworkEndpoint:
  return rdf_client_network.NetworkEndpoint.FromSerializedBytes(
      proto.SerializeToString()
  )


def ToProtoNetworkConnection(
    rdf: rdf_client_network.NetworkConnection,
) -> sysinfo_pb2.NetworkConnection:
  return rdf.AsPrimitiveProto()


def ToRDFNetworkConnection(
    proto: sysinfo_pb2.NetworkConnection,
) -> rdf_client_network.NetworkConnection:
  return rdf_client_network.NetworkConnection.FromSerializedBytes(
      proto.SerializeToString()
  )


def ToProtoNetworkAddress(
    rdf: rdf_client_network.NetworkAddress,
) -> jobs_pb2.NetworkAddress:
  return rdf.AsPrimitiveProto()


def ToRDFNetworkAddress(
    proto: jobs_pb2.NetworkAddress,
) -> rdf_client_network.NetworkAddress:
  return rdf_client_network.NetworkAddress.FromSerializedBytes(
      proto.SerializeToString()
  )


def ToProtoDNSClientConfiguration(
    rdf: rdf_client_network.DNSClientConfiguration,
) -> sysinfo_pb2.DNSClientConfiguration:
  return rdf.AsPrimitiveProto()


def ToRDFDNSClientConfiguration(
    proto: sysinfo_pb2.DNSClientConfiguration,
) -> rdf_client_network.DNSClientConfiguration:
  return rdf_client_network.DNSClientConfiguration.FromSerializedBytes(
      proto.SerializeToString()
  )


def ToProtoInterface(rdf: rdf_client_network.Interface) -> jobs_pb2.Interface:
  return rdf.AsPrimitiveProto()


def ToRDFInterface(proto: jobs_pb2.Interface) -> rdf_client_network.Interface:
  return rdf_client_network.Interface.FromSerializedBytes(
      proto.SerializeToString()
  )



#!/usr/bin/env python
"""This module will load all the configuration parameters."""

# pylint: disable=unused-import
from grr_response_core.config import acls
from grr_response_core.config import api
from grr_response_core.config import artifacts
from grr_response_core.config import build
from grr_response_core.config import client
from grr_response_core.config import config
from grr_response_core.config import contexts
from grr_response_core.config import data_store
from grr_response_core.config import gui
from grr_response_core.config import local
from grr_response_core.config import logging
from grr_response_core.config import server
from grr_response_core.config import test
# pylint: enable=unused-import

from grr_response_core.lib import config_lib

# By this time it's guaranteed that all configuration options
# and filters are imported and known to the config system.
CONFIG = config_lib._CONFIG  # pylint: disable=protected-access



#!/usr/bin/env python
from absl.testing import absltest

from grr_response_core.lib.util import iterator


class AssumeSingleTest(absltest.TestCase):

  def testSingle(self):
    items = iter([42])
    self.assertEqual(iterator.AssumeSingle(items), 42)

  def testEmpty(self):
    items = iter([])
    with self.assertRaises(iterator.NoYieldsError):
      iterator.AssumeSingle(items)

  def testMultiple(self):
    items = iter(["foo", "bar"])
    with self.assertRaises(iterator.TooManyYieldsError):
      iterator.AssumeSingle(items)


class CountedTest(absltest.TestCase):

  def testCountEmpty(self):
    empty = iterator.Counted(iter([]))
    self.assertEqual(empty.count, 0)

    with self.assertRaises(StopIteration):
      next(empty)

  def testCountSimple(self):
    items = iterator.Counted(iter(["foo", "bar", "baz"]))
    self.assertEqual(items.count, 0)
    self.assertEqual(next(items), "foo")
    self.assertEqual(items.count, 1)
    self.assertEqual(next(items), "bar")
    self.assertEqual(items.count, 2)
    self.assertEqual(next(items), "baz")
    self.assertEqual(items.count, 3)

    with self.assertRaises(StopIteration):
      next(items)

  def testCountInterrupted(self):

    def Generator():
      yield "foo"
      raise ValueError()
      yield "bar"  # pylint: disable=unreachable

    items = iterator.Counted(Generator())
    self.assertEqual(items.count, 0)
    self.assertEqual(next(items), "foo")
    self.assertEqual(items.count, 1)

    with self.assertRaises(ValueError):
      next(items)

    # The counter should not increase if the item was not retrieved.
    self.assertEqual(items.count, 1)

  def testResetEmpty(self):
    empty = iterator.Counted(iter([]))
    empty.Reset()
    self.assertEqual(empty.count, 0)

  def testResetSimple(self):
    items = iterator.Counted(iter(["foo", "bar", "baz"]))

    next(items)
    self.assertEqual(items.count, 1)

    items.Reset()
    self.assertEqual(items.count, 0)

    next(items)
    next(items)
    self.assertEqual(items.count, 2)

    items.Reset()
    self.assertEqual(items.count, 0)


class LookaheadTest(absltest.TestCase):

  def testEmptyNext(self):
    items = iterator.Lookahead(iter([]))
    with self.assertRaises(StopIteration):
      next(items)

  def testEmptyItem(self):
    items = iterator.Lookahead(iter([]))
    with self.assertRaises(ValueError):
      items.item  # pylint: disable=pointless-statement

  def testEmptyDone(self):
    items = iterator.Lookahead(iter([]))
    self.assertTrue(items.done)

  def testIterate(self):
    items = iterator.Lookahead(iter(["foo", "bar", "baz"]))
    self.assertEqual(list(items), ["foo", "bar", "baz"])

  def testItem(self):
    items = iterator.Lookahead(iter(["foo", "bar", "baz"]))
    self.assertEqual(items.item, "foo")
    next(items)
    self.assertEqual(items.item, "bar")
    next(items)
    self.assertEqual(items.item, "baz")

  def testDone(self):
    items = iterator.Lookahead(iter(["foo", "bar", "baz"]))
    self.assertFalse(items.done)
    next(items)
    self.assertFalse(items.done)
    next(items)
    self.assertFalse(items.done)
    next(items)
    self.assertTrue(items.done)


if __name__ == "__main__":
  absltest.main()



#!/usr/bin/env python
import io
import os
from unittest import mock

from absl.testing import absltest

from grr_response_core.lib.util import random


def WithUrandom(func):
  return mock.patch.object(os, "urandom", side_effect=func)


def WithRandomBuffer(values):
  return mock.patch.object(random, "_random_buffer", list(reversed(values)))


class UInt16Test(absltest.TestCase):

  @WithRandomBuffer([])
  @WithUrandom(lambda count: b"\x00" * count)
  def testMin(self, urandom):
    del urandom  # Unused.

    for _ in range(10000):
      if random.UInt16() != 0:
        self.assertEqual(random.UInt16(), 0)

  @WithRandomBuffer([])
  @WithUrandom(lambda count: b"\xff" * count)
  def testMax(self, urandom):
    del urandom  # Unused.

    for _ in range(10000):
      self.assertEqual(random.UInt16(), 2**16 - 1)

  @WithRandomBuffer([0xDEADDEAD, 0xBEEFBEEF])
  def testSpecific(self):
    self.assertEqual(random.UInt16(), 0xDEAD)
    self.assertEqual(random.UInt16(), 0xBEEF)


class PositiveUInt16Test(absltest.TestCase):

  @WithRandomBuffer([])
  @WithUrandom(io.BytesIO(b"\x00" * 10 * 1024 + b"\xff" * 10 * 1024).read)
  def testPositive(self, urandom):
    del urandom  # Unused.

    for _ in range(10):
      self.assertGreater(random.PositiveUInt16(), 0)


class UInt32Test(absltest.TestCase):

  @WithRandomBuffer([])
  @WithUrandom(lambda count: b"\x00" * count)
  def testMin(self, urandom):
    del urandom  # Unused.

    for _ in range(10000):
      self.assertEqual(random.UInt32(), 0)

  @WithRandomBuffer([])
  @WithUrandom(lambda count: b"\xff" * count)
  def testMax(self, urandom):
    del urandom  # Unused.

    for _ in range(10000):
      self.assertEqual(random.UInt32(), 2**32 - 1)

  @WithRandomBuffer([0xDEADBEEF, 0xBADDCAFE])
  def testSpecific(self):
    self.assertEqual(random.UInt32(), 0xDEADBEEF)
    self.assertEqual(random.UInt32(), 0xBADDCAFE)


class PositiveUInt32Test(absltest.TestCase):

  @WithRandomBuffer([])
  @WithUrandom(io.BytesIO(b"\x00" * 10 * 1024 + b"\xff" * 10 * 1024).read)
  def testPositive(self, urandom):
    del urandom  # Unused.

    for _ in range(10):
      self.assertGreater(random.PositiveUInt32(), 0)


class UInt64Test(absltest.TestCase):

  @WithRandomBuffer([])
  @WithUrandom(lambda count: b"\x00" * count)
  def testMin(self, urandom):
    del urandom  # Unused.

    for _ in range(10000):
      self.assertEqual(random.UInt64(), 0)

  @WithRandomBuffer([])
  @WithUrandom(lambda count: b"\xff" * count)
  def testMax(self, urandom):
    del urandom  # Unused.

    for _ in range(10000):
      self.assertEqual(random.UInt64(), 2**64 - 1)

  @WithRandomBuffer([0xDEADC0DE, 0xDEADB33F])
  def testSpecific(self):
    self.assertEqual(random.UInt64(), 0xDEADC0DEDEADB33F)


class Id64Test(absltest.TestCase):

  def testRandom(self):
    self.assertBetween(random.UInt64(), 2**32, 2**64 - 1)

  @WithRandomBuffer([])
  @WithUrandom(lambda count: b"\xff" * count)
  def testMax(self, urandom):
    del urandom  # Unused.

    for _ in range(10000):
      self.assertEqual(random.UInt64(), 2**64 - 1)

  @WithRandomBuffer([0xDEADC0DE, 0xDEADB33F])
  def testSpecific(self):
    self.assertEqual(random.Id64(), 0xDEADC0DEDEADB33F)


if __name__ == "__main__":
  absltest.main()



#!/usr/bin/env python
"""Utility functions for working with text representation of objects."""

import binascii

from grr_response_core.lib.util import precondition


def Asciify(data: bytes) -> str:
  """Turns given bytes to human-readable ASCII representation.

  All ASCII-representable bytes are turned into proper characters, whereas all
  other characters use escape sequences.

  Args:
    data: A byte sequence to convert.

  Returns:
    A human-readable representation of the given sequence.
  """
  precondition.AssertType(data, bytes)

  return repr(data)[2:-1]


def Hexify(data: bytes) -> str:
  """Turns given bytes to its hex representation.

  It works just like `binascii.hexlify` but always returns string objects rather
  than bytes. It also fixes the name of this function to avoid awkward "l".

  Args:
    data: A byte sequence to convert.

  Returns:
    A hex representation of the given data.
  """
  precondition.AssertType(data, bytes)
  return binascii.hexlify(data).decode("ascii")


def Unescape(string: str) -> str:
  """Evaluates string with escape sequences.

  Args:
    string: A string with escaped characters to unescape.

  Returns:
    An unescaped version of the input string.
  """
  precondition.AssertType(string, str)
  return string.encode("utf-8").decode("unicode_escape")



#!/usr/bin/env python
# Copyright 2011 Google Inc. All Rights Reserved.
"""Tests the query lexer."""

from absl import app

from grr_response_core.lib import lexer
from grr.test_lib import test_lib


class LexerTests(test_lib.GRRBaseTest):
  """Test the query language parser."""

  def testParser(self):
    """Test parenthesis precedence."""
    # First build an AST from the expression. (Deliberately include \r\n here):
    ast = lexer.SearchParser(""" ('file name' contains "foo") and (size > 100k
or date before "2011-10")""").Parse()

    self.assertEqual(ast.operator, "and")
    self.assertEqual(ast.args[0].attribute, "file name")
    self.assertEqual(ast.args[0].operator, "contains")
    self.assertEqual(ast.args[0].args[0], "foo")
    self.assertEqual(ast.args[1].operator, "or")
    self.assertEqual(ast.args[1].args[0].attribute, "size")
    self.assertEqual(ast.args[1].args[0].operator, ">")
    self.assertEqual(ast.args[1].args[0].args[0], "100k")
    self.assertEqual(ast.args[1].args[1].attribute, "date")
    self.assertEqual(ast.args[1].args[1].operator, "before")
    self.assertEqual(ast.args[1].args[1].args[0], "2011-10")

  def testParser2(self):
    """Test operator precedence."""
    # First build an AST from the expression. (Deliberately include \r\n here):
    ast = lexer.SearchParser(""" 'file name' contains "hello" and size > 100k
or 'file name' contains "foo" and date before "2011-10" """).Parse()

    self.assertEqual(ast.operator, "or")
    self.assertEqual(ast.args[0].operator, "and")
    self.assertEqual(ast.args[0].args[0].attribute, "file name")
    self.assertEqual(ast.args[0].args[0].operator, "contains")
    self.assertEqual(ast.args[0].args[0].args[0], "hello")
    self.assertEqual(ast.args[0].args[1].attribute, "size")
    self.assertEqual(ast.args[0].args[1].operator, ">")
    self.assertEqual(ast.args[0].args[1].args[0], "100k")

    self.assertEqual(ast.args[1].operator, "and")
    self.assertEqual(ast.args[1].args[0].attribute, "file name")
    self.assertEqual(ast.args[1].args[0].operator, "contains")
    self.assertEqual(ast.args[1].args[0].args[0], "foo")
    self.assertEqual(ast.args[1].args[1].attribute, "date")
    self.assertEqual(ast.args[1].args[1].operator, "before")
    self.assertEqual(ast.args[1].args[1].args[0], "2011-10")

  def testParser3(self):
    """Test quote escaping in strings."""
    # The following has an escaped quote.
    ast = lexer.SearchParser(r"""subject matches '"hello" \'world\''""").Parse()

    self.assertEqual(ast.operator, "matches")
    self.assertEqual(ast.args[0], """\"hello" 'world'""")

  def testFailedParser(self):
    """Test that the parser raises for invalid input."""
    # Some illegal expressions
    for expression in (
        """filename contains "foo""",  # Unterminated string
        """(filename contains "foo" """,  # Unbalanced parenthesis
        """filename contains foo or """,
    ):  # empty right expression
      parser = lexer.SearchParser(expression)
      self.assertRaises(lexer.ParseError, parser.Parse)


def main(argv):
  test_lib.main(argv)


if __name__ == "__main__":
  app.run(main)



#!/usr/bin/env python
"""Functions to convert strings between different case styles."""

import re


def SnakeToCamel(snake_str: str) -> str:
  """Convert a snake_case string representing one identifier to lowerCamelCase.

  The function uses a best-effort approach to convert the given string to a
  valid lowerCamelCase string, meaning that it converts even strings that use
  multiple consecutive underscores between words and/or that start/end with an
  underscore.

  Args:
    snake_str: A snake_case string representing a single identifier.

  Returns:
    A lowerCamelCase representation of the given string.
  """
  # Extract the words from the snake_case string.
  words = [word for word in snake_str.split("_") if word]
  if not words:
    return ""

  words[:] = map(str.lower, words)

  return words[0] + "".join(map(str.capitalize, words[1:]))


def CamelToSnake(lower_camel_str: str) -> str:
  """Convert a lowerCamelCase string representing one identifier to snake_case.

  The function uses a best-effort approach to convert the given string to a
  valid snake_case string, meaning that it converts even strings that start
  with a capital letter.

  Args:
    lower_camel_str: A lowerCamelCase string representing a single identifier.

  Returns:
    A snake_case representation of the given string.
  """
  camel_str_len = len(lower_camel_str)
  words = []
  i_word_start = 0

  for i_char in range(camel_str_len):
    if lower_camel_str[i_char].isupper():
      if i_word_start != i_char:  # Required for uppercase first letter case.
        words.append(lower_camel_str[i_word_start:i_char])
      i_word_start = i_char

  # Add the word at the end of the string, possibly the only word.
  words.append(lower_camel_str[i_word_start:camel_str_len])

  snake_str = "_".join(map(str.lower, words))

  # Remove duplicate underscores, if any.
  snake_str = re.sub(r"_+", "_", snake_str)

  # Remove the underscore from the start/end, if any.
  if snake_str.startswith("_"):
    snake_str = snake_str[1:]
  if snake_str.endswith("_"):
    snake_str = snake_str[:-1]

  return snake_str



#!/usr/bin/env python
"""Provides conversion functions to be used during RDFProtoStruct migration."""

from grr_response_core.lib.rdfvalues import dummy as rdf_dummy
from grr_response_proto import dummy_pb2


def ToProtoDummyRequest(rdf: rdf_dummy.DummyRequest) -> dummy_pb2.DummyRequest:
  return rdf.AsPrimitiveProto()


def ToRDFDummyRequest(proto: dummy_pb2.DummyRequest) -> rdf_dummy.DummyRequest:
  return rdf_dummy.DummyRequest.FromSerializedBytes(proto.SerializeToString())


def ToProtoDummyResult(rdf: rdf_dummy.DummyResult) -> dummy_pb2.DummyResult:
  return rdf.AsPrimitiveProto()


def ToRDFDummyResult(proto: dummy_pb2.DummyResult) -> rdf_dummy.DummyResult:
  return rdf_dummy.DummyResult.FromSerializedBytes(proto.SerializeToString())



#!/usr/bin/env python
"""Shut down windows hosts."""


import platform
import re


def NetshStaticIp(interface,
                  ip=u'127.0.0.9',
                  subnet=u'255.255.255.255',
                  gw=u'127.0.0.1'):
  """Changes interface to a statically set IP.

  Sets IP configs to local if no parameters passed.

  Args:
    interface: Name of the interface.
    ip: IP address.
    subnet: Subnet mask.
    gw: IP address of the default gateway.

  Returns:
    A tuple of stdout, stderr, exit_status.
  """
  args = [
      '/c', 'netsh', 'interface', 'ip', 'set', 'address', interface, 'static',
      ip, subnet, gw, '1'
  ]
  # pylint: disable=undefined-variable
  res = client_utils_common.Execute(
      'cmd', args, time_limit=-1, bypass_allowlist=True)
  return res


def DisableInterfaces(interface):
  """Tries to disable an interface.  Only works on Vista and 7.

  Args:
    interface: Name of the interface to disable.

  Returns:
    res which is a tuple of (stdout, stderr, exit_status, time_taken).
  """
  set_tested_versions = ['vista', '2008']
  set_args = ['/c', 'netsh', 'set', 'interface', interface, 'DISABLED']
  host_version = platform.platform().lower()
  for version in set_tested_versions:
    if host_version.find(version) != -1:
      # pylint: disable=undefined-variable
      res = client_utils_common.Execute(
          'cmd', set_args, time_limit=-1, bypass_allowlist=True)
      return res
  return ('', 'Command not available for this version.', 99, '')


def GetEnabledInterfaces():
  """Gives a list of enabled interfaces. Should work on all windows versions.

  Returns:
    interfaces: Names of interfaces found enabled.
  """
  interfaces = []
  show_args = ['/c', 'netsh', 'show', 'interface']
  # pylint: disable=undefined-variable
  res = client_utils_common.Execute(
      'cmd', show_args, time_limit=-1, bypass_allowlist=True)
  pattern = re.compile(r'\s*')
  for line in res[0].split('\r\n'):  # res[0] is stdout.
    interface_info = pattern.split(line)
    if 'Enabled' in interface_info:
      interfaces.extend(interface_info[-1:])
  return interfaces


def MsgUser(msg):
  """Sends a message to a user.

  Args:
    msg: Message to be displaied to user.

  Returns:
    res which is a tuple of (stdout, stderr, exit_status, time_taken).
  """
  msg_tested_versions = ['xp', 'vista', '2008', '2003']
  msg_args = ['/c', '%SystemRoot%\\System32\\msg.exe', '*', '/TIME:0']
  host_version = platform.platform().lower()
  if not msg:
    return ('Command not ran.', 'Empty message.', -1)
  else:
    msg_args.extend([msg])
  for version in msg_tested_versions:
    if host_version.find(version) != -1:
      # pylint: disable=undefined-variable
      res = client_utils_common.Execute(
          'cmd', msg_args, time_limit=-1, bypass_allowlist=True)
      return res
  return ('', 'Command not available for this version.', -1)


def main():
  return_str = {}
  # pylint: disable=g-bad-name
  MSG_STRING = ('***WARNING for Acme Corp Security***\n'
                'Your machine was found to be infected with a \n'
                'very scary virus. As a security measure we are \n'
                'shutting down your internet connection. Please \n'
                'call 0800-OHNOES immediately.')
  # pylint: disable=undefined-variable
  if 'msg' in py_args:
    return_str['msg'] = MsgUser(py_args['msg'])
  else:
    return_str['msg'] = MsgUser(MSG_STRING)

  for interface in GetEnabledInterfaces():
    if interface != 'Loopback' or interface != 'Internal':
      return_str[interface] = DisableInterfaces(interface)
      # Disabaling interface is not available.
      # Change interface config to be unroutable.
      if return_str[interface][2] == 99:
        if all([key in py_args for key in ['ip', 'subnet', 'gw']]):
          return_str[interface] = NetshStaticIp(
              interface, py_args['ip'], py_args['subnet'], py_args['gw'])
        else:
          return_str[interface] = NetshStaticIp(interface)

  # Build magic string.
  magic_list = []
  for key in return_str:
    stdout, stderr, exit_status, time_taken = return_str[key]
    key_str = '%s, %s, %s, %s, %s' % (key, stdout.encode('base64'),
                                      stderr.encode('base64'), exit_status,
                                      time_taken)
    magic_list.append(key_str)

  magic_return_str = ''.join(magic_list)  # pylint: disable=unused-variable


if __name__ == '__main__':
  main()



#!/usr/bin/env python
"""A module with RDF values wrapping container protobufs."""

from grr_response_core.lib.rdfvalues import structs as rdf_structs
from grr_response_proto import containers_pb2


class ListContainersOutput(rdf_structs.RDFProtoStruct):
  """An RDF wrapper class for the `ListContainersOutput` proto."""

  protobuf = containers_pb2.ListContainersOutput
  rdf_deps = []


class ListContainersRequest(rdf_structs.RDFProtoStruct):
  """Request for the `ListContainers` client action."""

  protobuf = containers_pb2.ListContainersRequest
  rdf_deps = []


class ListContainersResult(rdf_structs.RDFProtoStruct):
  """Result for the `ListContainers` client action."""

  protobuf = containers_pb2.ListContainersResult
  rdf_deps = [ListContainersOutput]



#!/usr/bin/env python
"""Tests for windows paths detection logic."""

from absl import app

from grr_response_core.path_detection import windows
from grr.test_lib import test_lib


class RunDllExtractorTest(test_lib.GRRBaseTest):
  """Tests for RunDllExtractor."""

  def setUp(self):
    super().setUp()
    self.extractor = windows.RunDllExtractor()

  def testDoesNothingIfFirstComponentIsNotRunDll(self):
    """Test it does nothing if the first component is not 'rundll'."""
    self.assertEqual(self.extractor.Extract(["a", "b"]), [])

  def testReturnsSecondOutOfTwoComponentsIfFirstOneIsRunDll(self):
    """Test it returns second out of 2 components if the first is 'rundll'."""
    self.assertEqual(self.extractor.Extract(["rundll32.exe", "b"]), ["b"])

  def testRunDllCheckIsCaseInsensitive(self):
    """Test it returns second out of 2 components if the first is 'rundll'."""
    self.assertEqual(self.extractor.Extract(["rUndll32.EXE", "b"]), ["b"])

  def testReturnsAllComponentsExceptForTheFirstOneIfFirstOneIsRunDll(self):
    """Test it returns all components except for the first 'rundll'."""
    self.assertEqual(
        self.extractor.Extract(["rundll32.exe", "b", "c", "d"]), ["b c d"]
    )

  def testReturnsThirdOutOfThreeComponentsIfFirstTwoAreRunDll(self):
    """Test it returns 3rd out of 2 components if the first two are rundll."""
    self.assertEqual(
        self.extractor.Extract([r"C:\some", r"path\rundll32.exe", "b"]), ["b"]
    )

  def testStripsFunctionName(self):
    """Test it strips the function name."""
    self.assertEqual(
        self.extractor.Extract(["rundll32.exe", "b,FuncName"]), ["b"]
    )


class ExecutableExtractorTest(test_lib.GRRBaseTest):
  """Tests for ExecutableExtractor."""

  def setUp(self):
    super().setUp()
    self.extractor = windows.ExecutableExtractor()

  def testIgnoresPathWithoutExecutableExtensions(self):
    """Test it ignores a path without executable extensions."""
    self.assertEqual(self.extractor.Extract(["a", "b", "c"]), [])

  def testReturnsPathIfExecutableExtensionIsFound(self):
    """Test it returns a path if executable extension is found."""
    self.assertEqual(self.extractor.Extract(["a", "b.exe", "c"]), ["a b.exe"])

  def testExtractionIsCaseInsensitive(self):
    """Test the extraction is case insensitive."""
    self.assertEqual(self.extractor.Extract(["a", "b.ExE", "c"]), ["a b.ExE"])


class EnvVarsPostProcessorTest(test_lib.GRRBaseTest):
  """Tests for EnvVarsPostProcessorTest."""

  def testDoesNothingIfMappingIsEmpty(self):
    """Test it does nothing if the mapping is empty."""
    processor = windows.EnvVarsPostProcessor({})
    self.assertEqual(processor.Process("a"), ["a"])

  def testReplacesOneVariable(self):
    """Test it correctly replaces one variable."""
    processor = windows.EnvVarsPostProcessor({"foo": "bar"})
    self.assertEqual(
        processor.Process(r"C:\WINDOWS\%foo%\something"),
        [r"C:\WINDOWS\bar\something"],
    )

  def testReplacesTwoVariables(self):
    """Test it correctly replaces two variables."""
    processor = windows.EnvVarsPostProcessor({"foo": "bar", "blah": "blahblah"})
    self.assertEqual(
        processor.Process(r"C:\WINDOWS\%foo%\%blah%\something"),
        [r"C:\WINDOWS\bar\blahblah\something"],
    )

  def testVariableReplacementIsCaseInsensitive(self):
    """Test variable replacement is case insensitive."""
    processor = windows.EnvVarsPostProcessor({"foo": "bar"})
    self.assertEqual(
        processor.Process(r"C:\WINDOWS\%FoO%\something"),
        [r"C:\WINDOWS\bar\something"],
    )

  def testGeneratesMultipleReplacementsIfReplacementIsList(self):
    """Test it generates multiple replacements if replacement is a list."""
    processor = windows.EnvVarsPostProcessor({"foo": ["bar", "blah"]})
    self.assertEqual(
        set(processor.Process(r"C:\WINDOWS\%foo%\something")),
        set([r"C:\WINDOWS\bar\something", r"C:\WINDOWS\blah\something"]),
    )

  def testVariableValueIsStableInASinglePath(self):
    """Test it keeps variable value stable in a single path."""
    processor = windows.EnvVarsPostProcessor({"foo": ["bar", "blah"]})
    self.assertEqual(
        set(processor.Process(r"C:\WINDOWS\%foo%\%foo%\something")),
        set(
            [r"C:\WINDOWS\bar\bar\something", r"C:\WINDOWS\blah\blah\something"]
        ),
    )

  def testGeneratesProductIfTwoReplacementsHaveMultipleValues(self):
    """Test it generates a product if two replacements have multiple values."""
    processor = windows.EnvVarsPostProcessor(
        {"foo": ["bar1", "bar2"], "blah": ["blah1", "blah2"]}
    )
    self.assertEqual(
        set(processor.Process(r"C:\WINDOWS\%foo%\%blah%\something")),
        set([
            r"C:\WINDOWS\bar1\blah1\something",
            r"C:\WINDOWS\bar1\blah2\something",
            r"C:\WINDOWS\bar2\blah1\something",
            r"C:\WINDOWS\bar2\blah2\something",
        ]),
    )

  def testReplacesSystemRootPrefixWithSystemRootVariable(self):
    """Test it replaces system root prefix with a system root variable."""
    processor = windows.EnvVarsPostProcessor({"systemroot": "blah"})
    self.assertEqual(
        processor.Process(r"\SystemRoot\foo\bar"), [r"blah\foo\bar"]
    )

  def testReplacesSystem32PrefixWithSystemRootVariable(self):
    """Test it replaces system32 prefix with a system root variable."""
    processor = windows.EnvVarsPostProcessor({"systemroot": "blah"})
    self.assertEqual(
        processor.Process(r"System32\foo\bar"), [r"blah\system32\foo\bar"]
    )


class WindowsRegistryExecutablePathsDetectorTest(test_lib.GRRBaseTest):
  """Tests for CreateWindowsRegistryExecutablePathsDetector() detector."""

  def testExtractsPathsFromNonRunDllStrings(self):
    """Test it extracts paths from non-rundll strings."""
    fixture = [
        (
            r"C:\Program Files\Realtek\Audio\blah.exe -s",
            r"C:\Program Files\Realtek\Audio\blah.exe",
        ),
        (
            r"'C:\Program Files\Realtek\Audio\blah.exe' -s",
            r"C:\Program Files\Realtek\Audio\blah.exe",
        ),
        (
            r"C:\Program Files\NVIDIA Corporation\nwiz.exe /quiet /blah",
            r"C:\Program Files\NVIDIA Corporation\nwiz.exe",
        ),
    ]

    for in_str, result in fixture:
      self.assertEqual(list(windows.DetectExecutablePaths([in_str])), [result])

  def testExctactsPathsFromRunDllStrings(self):
    """Test it extracts paths from rundll strings."""
    fixture = [
        (
            r"rundll32.exe C:\Windows\system32\advpack.dll,DelNodeRunDLL32",
            r"C:\Windows\system32\advpack.dll",
        ),
        (
            r"rundll32.exe 'C:\Program Files\Realtek\Audio\blah.exe',blah",
            r"C:\Program Files\Realtek\Audio\blah.exe",
        ),
        (
            r"'rundll32.exe' 'C:\Program Files\Realtek\Audio\blah.exe',blah",
            r"C:\Program Files\Realtek\Audio\blah.exe",
        ),
    ]

    for in_str, result in fixture:
      self.assertEqual(
          set(windows.DetectExecutablePaths([in_str])),
          set([result, "rundll32.exe"]),
      )

  def testReplacesEnvironmentVariable(self):
    """Test it replaces environment variables."""
    mapping = {
        "programfiles": r"C:\Program Files",
    }
    fixture = [
        (
            r"%ProgramFiles%\Realtek\Audio\blah.exe -s",
            r"C:\Program Files\Realtek\Audio\blah.exe",
        ),
        (
            r"'%ProgramFiles%\Realtek\Audio\blah.exe' -s",
            r"C:\Program Files\Realtek\Audio\blah.exe",
        ),
    ]

    for in_str, result in fixture:
      self.assertEqual(
          list(windows.DetectExecutablePaths([in_str], mapping)), [result]
      )

  def testReplacesEnvironmentVariablesWithMultipleMappings(self):
    """Test it replaces environment variables with multiple mappings."""

    mapping = {
        "appdata": [
            r"C:\Users\foo\Application Data",
            r"C:\Users\bar\Application Data",
        ]
    }

    fixture = [
        (
            r"%AppData%\Realtek\Audio\blah.exe -s",
            [
                r"C:\Users\foo\Application Data\Realtek\Audio\blah.exe",
                r"C:\Users\bar\Application Data\Realtek\Audio\blah.exe",
            ],
        ),
        (
            r"'%AppData%\Realtek\Audio\blah.exe' -s",
            [
                r"C:\Users\foo\Application Data\Realtek\Audio\blah.exe",
                r"C:\Users\bar\Application Data\Realtek\Audio\blah.exe",
            ],
        ),
    ]

    for in_str, result in fixture:
      self.assertEqual(
          set(windows.DetectExecutablePaths([in_str], mapping)), set(result)
      )


def main(argv):
  test_lib.main(argv)


if __name__ == "__main__":
  app.run(main)



#!/usr/bin/env python
import io
import os

from absl.testing import absltest
from cryptography import exceptions as crypto_exceptions
from cryptography.hazmat.primitives.ciphers import aead as crypto_aead

from grr_response_core.lib.util import aead


class EncryptDecryptTest(absltest.TestCase):

  def testReadWholeEmpty(self):
    self._testReadWhole(b"")

  def testReadWholeSmall(self):
    self._testReadWhole(b"foo")

  def testReadWholeRandom(self):
    self._testReadWhole(os.urandom(128))

  def testReadWholeRandomUneven(self):
    self._testReadWhole(os.urandom(131))

  def testReadWholeRandomLarge(self):
    self._testReadWhole(os.urandom(8 * 1024 * 1024))

  def testReadWholeRandomLargeUneven(self):
    self._testReadWhole(os.urandom(8 * 1024 * 1024 + 9))

  def _testReadWhole(self, data: bytes):  # pylint: disable=invalid-name
    key = os.urandom(32)

    encrypted = aead.Encrypt(io.BytesIO(data), key)
    decrypted = aead.Decrypt(encrypted, key)

    self.assertEqual(decrypted.read(), data)


class EncryptTest(absltest.TestCase):

  def testReadExact(self):
    key = os.urandom(32)
    data = os.urandom(1024)

    encrypted = aead.Encrypt(io.BytesIO(data), key)

    chunk_1 = encrypted.read(1)
    self.assertLen(chunk_1, 1)

    chunk_23 = encrypted.read(23)
    self.assertLen(chunk_23, 23)

    chunk_71 = encrypted.read(71)
    self.assertLen(chunk_71, 71)

    chunk_107 = encrypted.read(107)
    self.assertLen(chunk_107, 107)

    chunk_rest = encrypted.read()

    buf = io.BytesIO(chunk_1 + chunk_23 + chunk_71 + chunk_107 + chunk_rest)
    decrypted = aead.Decrypt(buf, key)

    self.assertEqual(decrypted.read(), data)


class DecryptTest(absltest.TestCase):

  def testReadExact(self):
    count = 2048
    key = os.urandom(32)

    buf = io.BytesIO(b"foobarbazquxnorf" * count)
    encrypted = aead.Encrypt(buf, key)
    decrypted = aead.Decrypt(encrypted, key)

    for _ in range(count):
      self.assertEqual(decrypted.read(3), b"foo")
      self.assertEqual(decrypted.read(3), b"bar")
      self.assertEqual(decrypted.read(3), b"baz")
      self.assertEqual(decrypted.read(3), b"qux")
      self.assertEqual(decrypted.read(4), b"norf")

    self.assertEqual(decrypted.read(), b"")

  def testIncorrectNonceLength(self):
    key = os.urandom(32)

    buf = io.BytesIO()

    nonce = os.urandom(aead._AEAD_NONCE_SIZE - 1)
    buf.write(nonce)
    buf.seek(0, io.SEEK_SET)

    with self.assertRaisesRegex(EOFError, "nonce length"):
      aead.Decrypt(buf, key).read()

  def testIncorrectTag(self):
    key = os.urandom(32)
    aesgcm = crypto_aead.AESGCM(key)

    buf = io.BytesIO()

    nonce = os.urandom(aead._AEAD_NONCE_SIZE)
    buf.write(nonce)
    buf.write(aesgcm.encrypt(nonce, b"foo", b"QUUX"))
    buf.seek(0, io.SEEK_SET)

    with self.assertRaises(crypto_exceptions.InvalidTag):
      aead.Decrypt(buf, key).read()

  def testIncorrectData(self):
    key = os.urandom(32)
    aesgcm = crypto_aead.AESGCM(key)

    buf = io.BytesIO()

    nonce = os.urandom(aead._AEAD_NONCE_SIZE)
    adata = aead._AEAD_ADATA_FORMAT.pack(0, True)
    buf.write(nonce)
    buf.write(aesgcm.encrypt(nonce, b"foo", adata))
    buf.getbuffer()[-1] ^= 0b10101010  # Corrupt last byte.
    buf.seek(0, io.SEEK_SET)

    with self.assertRaises(crypto_exceptions.InvalidTag):
      aead.Decrypt(buf, key).read()


if __name__ == "__main__":
  absltest.main()



#!/usr/bin/env python
"""Test protodict implementation.

An Dict is a generic dictionary implementation which has keys of type
string, and values of varying types. The Dict can be used to serialize
and transport arbitrary python dictionaries containing a limited set of value.

Dict objects behave generally like a dict (with __getitem__, items() and
an __iter__) method, but are serializable as an RDFProto.
"""

from collections import abc

from absl import app
from absl.testing import absltest

from grr_response_core.lib import rdfvalue
from grr_response_core.lib.rdfvalues import client_action as rdf_client_action
from grr_response_core.lib.rdfvalues import protodict as rdf_protodict
from grr_response_core.lib.rdfvalues import structs as rdf_structs
from grr_response_core.lib.rdfvalues import test_base as rdf_test_base
from grr_response_server.rdfvalues import flow_runner as rdf_flow_runner
from grr.test_lib import test_lib


class DictTest(rdf_test_base.RDFProtoTestMixin, test_lib.GRRBaseTest):
  """Test the Dict implementation."""

  rdfvalue_class = rdf_protodict.Dict

  def GenerateSample(self, number=0):
    return rdf_protodict.Dict(foo=number, bar="hello")

  def CheckRDFValue(self, value, sample):
    super().CheckRDFValue(value, sample)

    self.assertEqual(value.ToDict(), sample.ToDict())

  def CheckTestDict(self, test_dict, sample):
    for k, v in test_dict.items():
      # Test access through getitem.
      self.assertEqual(sample[k], v)

  def testEmbeddedDict(self):
    state = rdf_flow_runner.RequestState(data=rdf_protodict.Dict({"a": 1}))
    serialized = state.SerializeToBytes()
    deserialized = rdf_flow_runner.RequestState.FromSerializedBytes(serialized)
    self.assertEqual(deserialized.data, state.data)

  def testIsMapping(self):
    test_dict = rdf_protodict.Dict(a=1)
    self.assertIsInstance(test_dict, abc.Mapping)

  def testDictBehaviour(self):
    tested = rdf_protodict.Dict(a=1)

    now = rdfvalue.RDFDatetime.Now()
    tested["b"] = now

    self.assertEqual(tested["b"], now)
    self.assertEqual(tested["a"], 1)

    tested["b"] = rdfvalue.RDFURN("aff4:/users/")
    self.assertLen(tested, 2)
    self.assertEqual(tested["b"].SerializeToBytes(), b"aff4:/users")

  def testSerialization(self):
    test_dict = dict(
        key1=1,  # Integer.
        key2="foo",  # String.
        key3="\u4f60\u597d",  # Unicode.
        key5=rdfvalue.RDFDatetime.FromHumanReadable("2012/12/11"),  # RDFValue.
        key6=None,  # Support None Encoding.
        key7=rdf_structs.EnumNamedValue(5, name="Test"),  # Enums.
    )

    # Initialize through keywords.
    sample = rdf_protodict.Dict(**test_dict)
    self.CheckTestDict(test_dict, sample)

    # Initialize through dict.
    sample = rdf_protodict.Dict(test_dict)
    self.CheckTestDict(test_dict, sample)

    # Initialize through a serialized form.
    serialized = sample.SerializeToBytes()
    self.assertIsInstance(serialized, bytes)

    sample = rdf_protodict.Dict.FromSerializedBytes(serialized)
    self.CheckTestDict(test_dict, sample)

    # Convert to a dict.
    self.CheckTestDict(test_dict, sample.ToDict())

  def testNestedDicts(self):
    test_dict = dict(
        key1={"A": 1},
        key2=rdf_protodict.Dict({"A": 1}),
    )

    sample = rdf_protodict.Dict(**test_dict)
    self.CheckTestDict(test_dict, sample)
    self.CheckTestDict(test_dict, sample.ToDict())

  def testNestedDictsMultipleTypes(self):
    test_dict = dict(
        key1={"A": 1},
        key2=rdf_protodict.Dict({"A": 1}),
        key3=[1, 2, 3, [1, 2, [3]]],
        key4=[[], None, ["abc"]],
        key5=set([1, 2, 3]),
    )

    sample = rdf_protodict.Dict(**test_dict)
    self.CheckTestDict(test_dict, sample)
    to_dict = sample.ToDict()
    self.CheckTestDict(test_dict, to_dict)
    self.assertIsInstance(to_dict["key1"], dict)

  def testNestedDictsOpaqueTypes(self):

    class UnSerializable:
      pass

    test_dict = dict(
        key1={"A": 1},
        key2=rdf_protodict.Dict({"A": 1}),
        key3=[1, UnSerializable(), 3, [1, 2, [3]]],
        key4=[[], None, ["abc"]],
        key5=UnSerializable(),
        key6=["a", UnSerializable(), "b"],
    )

    self.assertRaises(TypeError, rdf_protodict.Dict, **test_dict)

    sample = rdf_protodict.Dict()
    for key, value in test_dict.items():
      sample.SetItem(key, value, raise_on_error=False)

    # Need to do some manual checking here since this is a lossy conversion.
    self.assertEqual(test_dict["key1"], sample["key1"])
    self.assertEqual(test_dict["key2"], sample["key2"])

    self.assertEqual(1, sample["key3"][0])
    self.assertIn("Unsupported type", sample["key3"][1])
    self.assertCountEqual(test_dict["key3"][2:], sample["key3"][2:])

    self.assertEqual(test_dict["key4"], sample["key4"])
    self.assertIn("Unsupported type", sample["key5"])
    self.assertEqual("a", sample["key6"][0])
    self.assertIn("Unsupported type", sample["key6"][1])
    self.assertEqual("b", sample["key6"][2])

  def testBool(self):
    sample = rdf_protodict.Dict(a=False)
    self.assertIs(sample["a"], False)

    sample = rdf_protodict.Dict(a=True)
    self.assertIs(sample["a"], True)

    sample = rdf_protodict.Dict(sample)
    self.assertIs(sample["a"], True)

    sample = rdf_protodict.Dict(a="true")
    self.assertEqual(sample["a"], "true")

  def testLegacyRDFBoolCanBeDeserialized(self):
    kv = rdf_protodict.KeyValue()
    kv.k.data = b"t"
    kv.v.rdf_value.name = b"RDFBool"
    kv.v.rdf_value.data = b"1"

    sample = rdf_protodict.Dict()
    sample._values[b"t"] = kv

    sample = rdf_protodict.Dict.FromSerializedBytes(sample.SerializeToBytes())
    self.assertIs(sample[b"t"], True)

  def testOverwriting(self):
    req = rdf_client_action.Iterator(client_state=rdf_protodict.Dict({"A": 1}))
    # There should be one element now.
    self.assertLen(list(req.client_state.items()), 1)

    req.client_state = rdf_protodict.Dict({"B": 2})
    # Still one element.
    self.assertLen(list(req.client_state.items()), 1)

    req.client_state = rdf_protodict.Dict({})

    # And now it's gone.
    self.assertEmpty(list(req.client_state.items()))


class DictSimpleTest(absltest.TestCase):

  def testToDictSimple(self):
    dct = {
        "foo": b"bar",
        b"baz": "quux",
    }
    self.assertEqual(rdf_protodict.Dict(dct).ToDict(), dct)

  def testToDictNestedDicts(self):
    dct = {
        "foo": {
            "bar": 42,
            "baz": 1337,
        },
        "quux": {
            "norf": 3.14,
            "thud": [4, 8, 15, 16, 23, 42],
        },
    }
    self.assertEqual(rdf_protodict.Dict(dct).ToDict(), dct)

  def testToDictNestedLists(self):
    dct = {
        "foo": [
            [42],
            [1, 3, 3, 7],
        ],
        "bar": [
            {
                "quux": [4, 8, 15],
                "thud": [16, 23],
            },
            {
                "norf": [42],
            },
        ],
    }
    self.assertEqual(rdf_protodict.Dict(dct).ToDict(), dct)


class AttributedDictTest(rdf_test_base.RDFValueTestMixin, test_lib.GRRBaseTest):
  """Test AttributedDictFile operations."""

  rdfvalue_class = rdf_protodict.AttributedDict

  def GenerateSample(self, number=0):
    return rdf_protodict.AttributedDict({"number": number})

  def testInitialize(self):
    arnie = {"target": "Sarah Connor", "mission": "Protect"}
    t800 = {"target": "Sarah Connor", "mission": "Terminate"}
    terminator = rdf_protodict.AttributedDict(arnie)
    self.assertEqual(terminator.GetItem("target"), "Sarah Connor")
    self.assertEqual(terminator.GetItem("mission"), "Protect")
    terminator = rdf_protodict.AttributedDict(t800)
    self.assertEqual(terminator.target, "Sarah Connor")
    self.assertEqual(terminator.mission, "Terminate")
    # We don't want a conflicted Terminator
    self.assertFalse(terminator.GetItem("happy_face"))

  def testAttributedDictSettingsAreAttr(self):
    t800 = {"target": "Sarah Connor", "mission": "Terminate"}
    terminator = rdf_protodict.AttributedDict(t800)
    self.assertEqual(terminator.target, "Sarah Connor")
    self.assertEqual(terminator.mission, "Terminate")


class AttributedDictSimpleTest(absltest.TestCase):

  def testInitFromStringKeyedDict(self):
    adict = rdf_protodict.AttributedDict({
        "foo": 42,
        "bar": b"quux",
        "baz": [4, 8, 15, 16, 23, 42],
    })

    self.assertEqual(adict.foo, 42)
    self.assertEqual(adict.bar, b"quux")
    self.assertEqual(adict.baz, [4, 8, 15, 16, 23, 42])

  # TODO: This behaviour should be removed once migration is done.
  def testInitFromBytestringKeyedDict(self):
    adict = rdf_protodict.AttributedDict({
        b"foo": 42,
        b"bar": b"quux",
        b"baz": [4, 8, 15, 16, 23, 42],
    })

    self.assertEqual(adict.foo, 42)
    self.assertEqual(adict.bar, b"quux")
    self.assertEqual(adict.baz, [4, 8, 15, 16, 23, 42])

  def testInitFromNonStringKeyedDictRaises(self):
    with self.assertRaises(TypeError):
      rdf_protodict.AttributedDict({
          1: "foo",
          2: "bar",
          3: "baz",
      })

  def testSetStringItem(self):
    adict = rdf_protodict.AttributedDict()
    adict["foo"] = 42
    adict["bar"] = b"quux"
    adict["baz"] = [4, 8, 15, 16, 23, 42]

    self.assertEqual(adict.foo, 42)
    self.assertEqual(adict.bar, b"quux")
    self.assertEqual(adict.baz, [4, 8, 15, 16, 23, 42])

  # TODO: This behaviour should be removed once migration is done.
  def testSetBytestringItem(self):
    adict = rdf_protodict.AttributedDict()
    adict[b"foo"] = 42
    adict[b"bar"] = b"quux"
    adict[b"baz"] = [4, 8, 15, 16, 23, 42]

    self.assertEqual(adict.foo, 42)
    self.assertEqual(adict.bar, b"quux")
    self.assertEqual(adict.baz, [4, 8, 15, 16, 23, 42])

  def testSetNonStringItemRaises(self):
    adict = rdf_protodict.AttributedDict()
    with self.assertRaises(TypeError):
      adict[42] = "foo"

  def testGetStringItem(self):
    adict = rdf_protodict.AttributedDict()
    adict.foo = 42
    adict.bar = b"quux"
    adict.baz = [4, 8, 15, 16, 23, 42]

    self.assertEqual(adict["foo"], 42)
    self.assertEqual(adict["bar"], b"quux")
    self.assertEqual(adict["baz"], [4, 8, 15, 16, 23, 42])

  # TODO: This behaviour should be removed once migration is done.
  def testGetBytestringItem(self):
    adict = rdf_protodict.AttributedDict()
    adict.foo = 42
    adict.bar = b"quux"
    adict.baz = [4, 8, 15, 16, 23, 42]

    self.assertEqual(adict[b"foo"], 42)
    self.assertEqual(adict[b"bar"], b"quux")
    self.assertEqual(adict[b"baz"], [4, 8, 15, 16, 23, 42])

  # TODO: This behaviour should be removed once migration is done.
  def testFromSerializedProtoDict(self):
    # In this test we use a non-attributed dict to force a serialization with
    # byte keys and then we deserialize it as attributed dict that should have
    # these attributes properly to unicode string keys.
    pdict = rdf_protodict.Dict()
    pdict[b"foo"] = 42
    pdict[b"bar"] = b"quux"
    pdict[b"baz"] = [4, 8, 15, 16, 23, 42]
    serialized = pdict.SerializeToBytes()

    adict = rdf_protodict.AttributedDict.FromSerializedBytes(serialized)
    self.assertEqual(adict.foo, 42)
    self.assertEqual(adict.bar, b"quux")
    self.assertEqual(adict.baz, [4, 8, 15, 16, 23, 42])

  # TODO: This behaviour should be removed once migration is done.
  def testToPrimitiveDict(self):
    # See rationale for using serialized non-attributed dict above.
    pdict = rdf_protodict.Dict()
    pdict[b"foo"] = 42
    pdict[b"bar"] = b"quux"
    pdict[b"baz"] = [4, 8, 15, 16, 23, 42]
    serialized = pdict.SerializeToBytes()

    adict = rdf_protodict.AttributedDict.FromSerializedBytes(serialized)

    dct = adict.ToDict()
    self.assertEqual(dct["foo"], 42)
    self.assertEqual(dct["bar"], b"quux")
    self.assertEqual(dct["baz"], [4, 8, 15, 16, 23, 42])

  def testNestedAssignment(self):
    adict = rdf_protodict.AttributedDict()

    adict["foo"] = {}
    adict["foo"]["bar"] = 42
    adict["foo"][b"baz"] = "Lorem ipsum."

    adict[b"quux"] = {}
    adict[b"quux"]["norf"] = [4, 8, 15, 16, 23, 42]
    adict[b"quux"][b"thud"] = 3.14

    self.assertEqual(adict.foo["bar"], 42)
    self.assertEqual(adict.foo[b"baz"], "Lorem ipsum.")
    self.assertEqual(adict.quux["norf"], [4, 8, 15, 16, 23, 42])
    self.assertEqual(adict.quux[b"thud"], 3.14)


class EmbeddedRDFValueTest(
    rdf_test_base.RDFProtoTestMixin, test_lib.GRRBaseTest
):
  rdfvalue_class = rdf_protodict.EmbeddedRDFValue

  def GenerateSample(self, number=0):
    return rdf_protodict.EmbeddedRDFValue(
        rdf_protodict.DataBlob(integer=number)
    )


def main(argv):
  # Run the full test suite
  test_lib.main(argv)


if __name__ == "__main__":
  app.run(main)



#!/usr/bin/env python
"""A module with utilities for dealing with temporary files and directories."""

import os
import platform
import shutil
import tempfile
from typing import Optional

from absl import flags

from grr_response_core.lib.util import precondition

FLAGS = flags.FLAGS


def _TestTempRootPath() -> Optional[str]:
  """Returns a default root path for storing temporary files during tests."""
  # `TEST_TMPDIR` and `FLAGS.test_tmpdir` are only defined only for test
  # environments. For non-test code, we use the default temporary directory.
  test_tmpdir = os.environ.get("TEST_TMPDIR")
  if test_tmpdir is None and hasattr(FLAGS, "test_tmpdir"):
    test_tmpdir = FLAGS.test_tmpdir

  if test_tmpdir is not None:
    try:
      os.makedirs(test_tmpdir)
    except FileExistsError:
      pass

  # TODO(hanuszczak): Investigate whether this check still makes sense.
  if platform.system() == "Windows":
    return None

  return test_tmpdir


def TempDirPath(suffix: str = "", prefix: str = "tmp") -> str:
  """Creates a temporary directory based on the environment configuration.

  The directory will be placed in folder as specified by the `TEST_TMPDIR`
  environment variable if available or fallback to `FLAGS.test_tmpdir` if
  provided or just use Python's default.

  Args:
    suffix: A suffix to end the directory name with.
    prefix: A prefix to begin the directory name with.

  Returns:
    An absolute path to the created directory.
  """
  precondition.AssertType(suffix, str)
  precondition.AssertType(prefix, str)

  return tempfile.mkdtemp(suffix=suffix, prefix=prefix, dir=_TestTempRootPath())


def TempFilePath(
    suffix: str = "",
    prefix: str = "tmp",
    dir: Optional[str] = None,  # pylint: disable=redefined-builtin
) -> str:
  """Creates a temporary file based on the environment configuration.

  If no directory is specified the file will be placed in folder as specified by
  the `TEST_TMPDIR` environment variable if available or fallback to
  `FLAGS.test_tmpdir` if provided or just use Python's default.

  If directory is specified it must be part of the default test temporary
  directory.

  Args:
    suffix: A suffix to end the file name with.
    prefix: A prefix to begin the file name with.
    dir: A directory to place the file in.

  Returns:
    An absolute path to the created file.

  Raises:
    ValueError: If the specified directory is not part of the default test
        temporary directory.
  """
  precondition.AssertType(suffix, str)
  precondition.AssertType(prefix, str)
  precondition.AssertOptionalType(dir, str)

  root = _TestTempRootPath()
  if not dir:
    dir = root
  elif root and not os.path.commonprefix([dir, root]):
    raise ValueError("path '%s' must start with '%s'" % (dir, root))

  # `mkstemp` returns an open descriptor for the file. We don't care about it as
  # we are only concerned with the path, but we need to close it first or else
  # the file will remain open until the garbage collectors steps in.
  desc, path = tempfile.mkstemp(suffix=suffix, prefix=prefix, dir=dir)
  os.close(desc)
  return path


class AutoTempDirPath:
  """Creates a temporary directory based on the environment configuration.

  The directory will be placed in folder as specified by the `TEST_TMPDIR`
  environment variable if available or fallback to `FLAGS.test_tmpdir` if
  provided or just use Python's default.

  This object is a context manager and the directory is automatically removed
  when it goes out of scope.

  Returns:
    An absolute path to the created directory.
  """

  def __init__(
      self,
      suffix: str = "",
      prefix: str = "tmp",
      remove_non_empty: bool = False,
  ):
    """Creates a temporary directory based on the environment configuration.

    Args:
      suffix: A suffix to end the directory name with.
      prefix: A prefix to begin the directory name with.
      remove_non_empty: If set to `True` the directory removal will succeed even
        if it is not empty.
    """
    precondition.AssertType(suffix, str)
    precondition.AssertType(prefix, str)
    precondition.AssertType(remove_non_empty, bool)

    self.suffix = suffix
    self.prefix = prefix
    self.remove_non_empty = remove_non_empty

  def __enter__(self) -> str:
    self.path = TempDirPath(suffix=self.suffix, prefix=self.prefix)
    return self.path

  def __exit__(self, exc_type, exc_value, traceback):
    del exc_type  # Unused.
    del exc_value  # Unused.
    del traceback  # Unused.

    if self.remove_non_empty:
      shutil.rmtree(self.path)
    else:
      os.rmdir(self.path)


class AutoTempFilePath:
  """Creates a temporary file based on the environment configuration.

  If no directory is specified the file will be placed in folder as specified by
  the `TEST_TMPDIR` environment variable if available or fallback to
  `FLAGS.test_tmpdir` if provided or just default to Python's default.

  If directory is specified it must be part of the default test temporary
  directory.

  This object is a context manager and the associated file is automatically
  removed when it goes out of scope.

  Returns:
    An absolute path to the created file.

  Raises:
    ValueError: If the specified directory is not part of the default test
        temporary directory.
  """

  def __init__(
      self,
      suffix: str = "",
      prefix: str = "tmp",
      dir: Optional[str] = None,  # pylint: disable=redefined-builtin
  ):
    """Creates a temporary file based on the environment configuration.

    Args:
      suffix: A suffix to end the file name with.
      prefix: A prefix to begin the file name with.
      dir: A directory to place the file in.
    """
    precondition.AssertType(suffix, str)
    precondition.AssertType(prefix, str)
    precondition.AssertType(suffix, str)
    precondition.AssertOptionalType(dir, str)

    self.suffix = suffix
    self.prefix = prefix
    self.dir = dir

  def __enter__(self) -> str:
    self.path = TempFilePath(
        suffix=self.suffix, prefix=self.prefix, dir=self.dir
    )
    return self.path

  def __exit__(self, exc_type, exc_value, traceback):
    del exc_type  # Unused.
    del exc_value  # Unused.
    del traceback  # Unused.

    os.remove(self.path)



#!/usr/bin/env python
"""A module with utilities for dealing with context managers."""
from collections.abc import Sequence
import contextlib
from typing import Generic, TypeVar

_T = TypeVar("_T")


class MultiContext(
    contextlib.AbstractContextManager[Sequence[_T]], Generic[_T]
):
  """A context managers that sequences multiple context managers.

  This is similar to the monadic `sequence` operator: it takes a list of context
  managers, enters each of them and yields list of values that the managers
  yield.

  One possible scenario where this class comes in handy is when one needs to
  open multiple files.
  """

  def __init__(
      self, managers: Sequence[contextlib.AbstractContextManager[_T]]
  ) -> None:
    self._managers = managers

  def __enter__(self) -> Sequence[_T]:
    values = []
    for manager in self._managers:
      value = manager.__enter__()
      values.append(value)
    return values

  def __exit__(self, exc_type, exc_value, traceback):
    for manager in self._managers:
      manager.__exit__(exc_type, exc_value, traceback)



#!/usr/bin/env python
"""Configuration parameters for the data stores."""


from grr_response_core.lib import config_lib

config_lib.DEFINE_integer("Datastore.maximum_blob_size", 512 * 1024,
                          "Maximum blob size we may store in the datastore.")

config_lib.DEFINE_string("Blobstore.implementation", "DbBlobStore",
                         "Blob storage subsystem to use.")

config_lib.DEFINE_string("Database.implementation", "",
                         "Relational database system to use.")

# MySQL configuration.
config_lib.DEFINE_string("Mysql.host", "localhost",
                         "The MySQL server hostname.")

config_lib.DEFINE_integer("Mysql.port", 0, "The MySQL server port.")

config_lib.DEFINE_string(
    "Mysql.username",
    default="root",
    help="The user to connect to the database.")

config_lib.DEFINE_string(
    "Mysql.password",
    default="",
    help="The password to connect to the database.")

config_lib.DEFINE_string(
    "Mysql.database", default="grr_db", help="Name of the database to use.")

config_lib.DEFINE_integer(
    "Mysql.conn_pool_max",
    default=10,
    help="The maximum number of open connections to keep available in the pool."
)

config_lib.DEFINE_integer(
    "Mysql.flow_processing_threads_min",
    default=1,
    help="The minimum number of flow-processing worker threads.",
)

config_lib.DEFINE_integer(
    "Mysql.flow_processing_threads_max",
    default=20,
    help="The maximum number of flow-processing worker threads.",
)

config_lib.DEFINE_string(
    "Mysql.migrations_dir", "%(grr_response_server/databases/mysql_migrations@"
    "grr-response-server|resource)", "Folder with MySQL migrations files.")

# Support for MySQL SSL connections.

config_lib.DEFINE_string(
    "Mysql.client_key_path",
    default="",
    help="The path name of the client private key file.")

config_lib.DEFINE_string(
    "Mysql.client_cert_path",
    default="",
    help="The path name of the client public key certificate file.")

config_lib.DEFINE_string(
    "Mysql.ca_cert_path",
    default="",
    help="The path name of the Certificate Authority (CA) certificate file. "
    "This option, if used, must specify the same certificate used by the "
    "server.")

# Legacy MySQLAdvancedDataStore used as AFF4 backend.
config_lib.DEFINE_string(
    "Mysql.database_name", default="grr", help="Deprecated.")

config_lib.DEFINE_string("Mysql.table_name", default="aff4", help="Deprecated.")

config_lib.DEFINE_string(
    "Mysql.database_username", default="root", help="Deprecated.")

config_lib.DEFINE_string(
    "Mysql.database_password", default="", help="Deprecated.")

config_lib.DEFINE_integer("Mysql.conn_pool_min", 5, help="Deprecated.")

config_lib.DEFINE_integer("Mysql.max_connect_wait", 600, help="Deprecated.")

config_lib.DEFINE_integer(
    "Mysql.max_query_size", 8 * 1024 * 1024, help="Deprecated.")

config_lib.DEFINE_integer(
    "Mysql.max_values_per_query", 10000, help="Deprecated.")

config_lib.DEFINE_integer("Mysql.max_retries", 10, help="Deprecated.")

# GCS blobstore config
config_lib.DEFINE_string(
    "Blobstore.gcs.project",
    default=None,
    help=(
        "GCP project where GCS.blobstore_bucket is located. Only used when "
        "Blobstore.implementation is GCSBlobStore."
    ),
)
config_lib.DEFINE_string(
    "Blobstore.gcs.bucket",
    default=None,
    help="GCS bucket to use when Blobstore.implementation is GCSBlobStore.",
)
config_lib.DEFINE_string(
    "Blobstore.gcs.blob_prefix",
    default="",
    help=(
        "An optional prefix to prepend to blob file names uploaded to GCS. "
        "This can be used to simulate a 'subdirectory' for GCS blob storage. "
        "I.e. if a value ending in '/' is set, such as 'blob_subdir/'. "
        "Only used when Blobstore.implementation is GCSBlobStore."
    ),
)



#!/usr/bin/env python
"""A module with filesystem-related utility functions and classes."""

import array
import os
import platform
import stat
from typing import NamedTuple, Optional

from grr_response_core.lib.util import precondition


class Stat:
  """A wrapper around standard `os.[l]stat` function.

  The standard API for using `stat` results is very clunky and unpythonic.
  This is an attempt to create a more familiar and consistent interface to make
  the code look cleaner.

  Moreover, standard `stat` does not properly support extended flags - even
  though the documentation mentions that `stat.st_flags` should work on macOS
  and Linux it works only on macOS and raises an error on Linux (and Windows).
  This class handles that and fetches these flags lazily (as it can be costly
  operation on Linux).
  """

  @classmethod
  def FromPath(cls, path: str, follow_symlink: bool = True) -> "Stat":
    """Returns stat information about the given OS path, calling os.[l]stat.

    Args:
      path: A path to perform `stat` on.
      follow_symlink: True if `stat` of a file that a symlink points to should
        be returned instead of the symlink itself. For non-symlinks this setting
        has no effect.

    Returns:
      Stat instance, with information about the given path.
    """
    # Note that we do not add type assertion for `path` here. The reason is that
    # many of the existing system calls (e.g. `os.listdir`) return results as
    # bytestrings in Python 2. This is fine because it also means that they also
    # accept bytestring paths as arguments in Python 2 (e.g. `os.stat`). Having
    # consistent types in both versions is certainly desired but it might be too
    # much work for too little benefit.
    precondition.AssertType(follow_symlink, bool)

    if follow_symlink:
      stat_obj = os.stat(path)
    else:
      stat_obj = os.lstat(path)

    try:
      target = os.readlink(path)
    # `os.readlink` raises `ValueError` on Windows and `OSError` on UNIX.
    except (OSError, ValueError):
      target = None

    return cls(path=path, stat_obj=stat_obj, symlink_target=target)

  def __init__(
      self,
      path: str,
      stat_obj: os.stat_result,
      symlink_target: Optional[str] = None,
  ) -> None:
    """Wrap an existing stat result in a `filesystem.Stat` instance.

    Args:
      path: the path of `stat_obj`.
      stat_obj: an instance of os.stat_result with information about `path`.
      symlink_target: Path of the original file that symlink refers to.
    """
    self._path = path
    self._stat = stat_obj
    self._symlink_target = symlink_target
    self._flags_linux = None
    self._flags_osx = None

  def GetRaw(self) -> os.stat_result:
    return self._stat

  def GetPath(self) -> str:
    return self._path

  def GetLinuxFlags(self) -> int:
    if self._flags_linux is None:
      self._flags_linux = self._FetchLinuxFlags()
    return self._flags_linux

  def GetOsxFlags(self) -> int:
    if self._flags_osx is None:
      self._flags_osx = self._FetchOsxFlags()
    return self._flags_osx

  def GetSize(self) -> int:
    return self._stat.st_size

  def GetAccessTime(self) -> int:
    # st_atime_ns is a higher-precision version of st_atime. Use it if it's
    # present.
    if self._stat.st_atime_ns is not None:
      return _NanosecondsToMicroseconds(self._stat.st_atime_ns)
    else:
      return _SecondsToMicroseconds(self._stat.st_atime.AsSecondsSinceEpoch())

  def GetModificationTime(self) -> int:
    # st_mtime_ns is a higher-precision version of st_mtime. Use it if it's
    # present.
    if self._stat.st_mtime_ns is not None:
      return _NanosecondsToMicroseconds(self._stat.st_mtime_ns)
    else:
      return _SecondsToMicroseconds(self._stat.st_mtime.AsSecondsSinceEpoch())

  def GetChangeTime(self) -> int:
    # st_ctime_ns is a higher-precision version of st_ctime. Use it if it's
    # present.
    if self._stat.st_ctime_ns is not None:
      return _NanosecondsToMicroseconds(self._stat.st_ctime_ns)
    else:
      return _SecondsToMicroseconds(self._stat.st_ctime.AsSecondsSinceEpoch())

  def GetDevice(self) -> int:
    return self._stat.st_dev

  def GetSymlinkTarget(self) -> Optional[str]:
    return self._symlink_target

  def IsDirectory(self) -> bool:
    return stat.S_ISDIR(self._stat.st_mode)

  def IsRegular(self) -> bool:
    return stat.S_ISREG(self._stat.st_mode)

  def IsSocket(self) -> bool:
    return stat.S_ISSOCK(self._stat.st_mode)

  def IsSymlink(self) -> bool:
    return stat.S_ISLNK(self._stat.st_mode)

  # http://manpages.courier-mta.org/htmlman2/ioctl_list.2.html
  FS_IOC_GETFLAGS = 0x80086601

  def _FetchLinuxFlags(self) -> int:
    """Fetches Linux extended file flags."""
    if platform.system() != "Linux":
      return 0

    # Since we open a file in the next step we do not want to open a symlink.
    # `lsattr` returns an error when trying to check flags of a symlink, so we
    # assume that symlinks cannot have them.
    if self.IsSymlink():
      return 0

    # Some files (e.g. sockets) cannot be opened. For these we do not really
    # care about extended flags (they should have none). `lsattr` does not seem
    # to support such cases anyway. It is also possible that a file has been
    # deleted (because this method is used lazily).
    try:
      fd = os.open(self._path, os.O_RDONLY)
    except (IOError, OSError):
      return 0

    try:
      # This import is Linux-specific.
      import fcntl  # pylint: disable=g-import-not-at-top

      buf = array.array("l", [0])
      # TODO(user):pytype: incorrect type spec for fcntl.ioctl
      # pytype: disable=wrong-arg-types
      fcntl.ioctl(fd, self.FS_IOC_GETFLAGS, buf)
      # pytype: enable=wrong-arg-types
      return buf[0]
    except (IOError, OSError):
      # File system does not support extended attributes.
      return 0
    finally:
      os.close(fd)

  def _FetchOsxFlags(self) -> int:
    """Fetches macOS extended file flags."""
    if platform.system() != "Darwin":
      return 0

    return self._stat.st_flags  # pytype: disable=attribute-error


class StatCache:
  """An utility class for avoiding unnecessary syscalls to `[l]stat`.

  This class is useful in situations where manual bookkeeping of stat results
  in order to prevent extra system calls becomes tedious and complicates control
  flow. This class makes sure that no unnecessary system calls are made and is
  smart enough to cache symlink results when a file is not a symlink.
  """

  _Key = NamedTuple("_Key", (("path", str), ("follow_symlink", bool)))  # pylint: disable=invalid-name

  def __init__(self):
    self._cache: dict[StatCache._Key, Stat] = {}

  def Get(self, path: str, follow_symlink: bool = True) -> Stat:
    """Stats given file or returns a cached result if available.

    Args:
      path: A path to the file to perform `stat` on.
      follow_symlink: True if `stat` of a file that a symlink points to should
        be returned instead of the symlink itself. For non-symlinks this setting
        has no effect.

    Returns:
      `Stat` object corresponding to the given path.
    """
    key = self._Key(path=path, follow_symlink=follow_symlink)
    try:
      return self._cache[key]
    except KeyError:
      value = Stat.FromPath(path, follow_symlink=follow_symlink)
      self._cache[key] = value

      # If we are not following symlinks and the file is a not symlink then
      # the stat result for this file stays the same even if we want to follow
      # symlinks.
      if not follow_symlink and not value.IsSymlink():
        self._cache[self._Key(path=path, follow_symlink=True)] = value

      return value


def _NanosecondsToMicroseconds(ns: int) -> int:
  """Converts nanoseconds to microseconds."""
  return ns // 1000


def _SecondsToMicroseconds(ns: float) -> int:
  """Converts seconds to microseconds."""
  return int(ns * 1e6)



#!/usr/bin/env python
"""Test standard RDFValues."""

from absl import app

from grr_response_core.lib.rdfvalues import standard as rdf_standard
from grr_response_core.lib.rdfvalues import test_base as rdf_test_base
from grr.test_lib import test_lib


class URITests(rdf_test_base.RDFValueTestMixin, test_lib.GRRBaseTest):
  """Test URI proto."""

  rdfvalue_class = rdf_standard.URI

  def GenerateSample(self, number=0):
    return rdf_standard.URI(transport="http", host="%s.example.com" % number)

  def testURI(self):
    sample = rdf_standard.URI(
        transport="http",
        host="google.com",
        path="/index",
        query="q=hi",
        fragment="anchor1",
    )
    self.assertEqual(sample.transport, "http")
    self.assertEqual(sample.host, "google.com")
    self.assertEqual(sample.path, "/index")
    self.assertEqual(sample.query, "q=hi")
    self.assertEqual(sample.fragment, "anchor1")

  def testHumanReadable(self):
    url = "http://google.com:443/search?query=hi#anchor2"
    sample = rdf_standard.URI.FromHumanReadable(url)

    self.assertEqual(sample.transport, "http")
    self.assertEqual(sample.host, "google.com:443")
    self.assertEqual(sample.path, "/search")
    self.assertEqual(sample.query, "query=hi")
    self.assertEqual(sample.fragment, "anchor2")

    self.assertEqual(sample.SerializeToHumanReadable(), url)

  def testByteString(self):
    raw_uri = "http://gógiel.pl:1337/znajdź?frazę=🦋#nagłówek"

    uri = rdf_standard.URI.FromSerializedBytes(raw_uri.encode("utf-8"))

    self.assertEqual(uri.transport, "http")
    self.assertEqual(uri.host, "gógiel.pl:1337")
    self.assertEqual(uri.path, "/znajdź")
    self.assertEqual(uri.query, "frazę=🦋")
    self.assertEqual(uri.fragment, "nagłówek")

    self.assertEqual(uri.FromSerializedBytes(uri.SerializeToBytes()), uri)


def main(argv):
  # Run the full test suite
  test_lib.main(argv)


if __name__ == "__main__":
  app.run(main)



#!/usr/bin/env python
"""RDFValue instances related to the statistics collection."""

import bisect

from grr_response_core.lib.rdfvalues import client_stats as rdf_client_stats
from grr_response_core.lib.rdfvalues import structs as rdf_structs
from grr_response_proto import analysis_pb2
from grr_response_proto import jobs_pb2


class Distribution(rdf_structs.RDFProtoStruct):
  """Statistics values for events - i.e. things that take time."""

  protobuf = jobs_pb2.Distribution

  def __init__(self, initializer=None, bins=None):
    if initializer and bins:
      raise ValueError(
          "Either 'initializer' or 'bins' arguments can be specified."
      )

    super().__init__(initializer=initializer)

    if bins:
      self.bins = [-float("inf")] + bins
    else:
      self.bins = []

    self.heights = [0] * len(self.bins)

  def Record(self, value):
    """Records given value."""
    self.sum += value
    self.count += 1

    pos = bisect.bisect(self.bins, value) - 1
    if pos < 0:
      pos = 0
    elif pos == len(self.bins):
      pos = len(self.bins) - 1

    self.heights[pos] += 1

  @property
  def bins_heights(self):
    return dict(zip(self.bins, self.heights))


class MetricFieldDefinition(rdf_structs.RDFProtoStruct):
  """Metric field definition."""

  protobuf = jobs_pb2.MetricFieldDefinition


class MetricMetadata(rdf_structs.RDFProtoStruct):
  """Metric metadata for a particular metric."""

  protobuf = jobs_pb2.MetricMetadata
  rdf_deps = [
      MetricFieldDefinition,
  ]

  def DefaultValue(self):
    if self.value_type == self.ValueType.INT:
      return 0
    elif self.value_type == self.ValueType.FLOAT:
      return 0.0
    elif self.value_type == self.ValueType.DISTRIBUTION:
      return Distribution()
    else:
      raise ValueError("Illegal value type: {!r}".format(self.value_type))


class StatsHistogramBin(rdf_structs.RDFProtoStruct):
  protobuf = jobs_pb2.StatsHistogramBin


class StatsHistogram(rdf_structs.RDFProtoStruct):
  """Histogram with a user-provided set of bins."""

  protobuf = jobs_pb2.StatsHistogram
  rdf_deps = [
      StatsHistogramBin,
  ]


class RunningStats(rdf_structs.RDFProtoStruct):
  """Class for collecting running stats: mean, stddev and histogram data."""

  protobuf = jobs_pb2.RunningStats
  rdf_deps = [
      StatsHistogram,
  ]


class ClientResourcesStats(rdf_structs.RDFProtoStruct):
  """RDF value representing clients' resources usage statistics for hunts."""

  protobuf = jobs_pb2.ClientResourcesStats
  rdf_deps = [
      rdf_client_stats.ClientResources,
      RunningStats,
  ]


class Sample(rdf_structs.RDFProtoStruct):
  """A Graph sample is a single data point."""

  protobuf = analysis_pb2.Sample


class SampleFloat(rdf_structs.RDFProtoStruct):
  """A Graph float data point."""

  protobuf = analysis_pb2.SampleFloat


class Graph(rdf_structs.RDFProtoStruct):
  """A Graph is a collection of sample points."""

  protobuf = analysis_pb2.Graph
  rdf_deps = [
      Sample,
  ]

  def Append(self, **kwargs):
    self.data.Append(**kwargs)

  def __len__(self):
    return len(self.data)

  def __bool__(self):
    return bool(self.data)

  def __getitem__(self, item):
    return Sample(self.data[item])

  def __iter__(self):
    for x in self.data:
      yield Sample(x)


class ClientGraphSeries(rdf_structs.RDFProtoStruct):
  """A collection of graphs for a single client-report type."""

  protobuf = analysis_pb2.ClientGraphSeries
  rdf_deps = [
      Graph,
  ]



#!/usr/bin/env python
"""Semantic protocol buffers can be created from proto2 .proto files.

For maintaining inter-operatibility with primitive protocol buffer
implementations, we can parse the field descriptors created by the standard
Google proto implementation, and generate Semantic proto descriptors.

This file contains interoperability code with the Google protocol buffer
library.
"""

import inspect
import logging

from grr_response_core.lib import rdfvalue
from grr_response_core.lib import type_info
from grr_response_proto import semantic_pb2


class Error(Exception):
  """Base error class for this module."""


class ProtobufNameMustMatchClassOrParentClassError(Error):
  pass


# Field types present in the proto2 field descriptors.
TYPE_DOUBLE = 1
TYPE_FLOAT = 2
TYPE_INT64 = 3
TYPE_UINT64 = 4
TYPE_INT32 = 5
TYPE_FIXED64 = 6
TYPE_FIXED32 = 7
TYPE_BOOL = 8
TYPE_STRING = 9
TYPE_GROUP = 10
TYPE_MESSAGE = 11
TYPE_BYTES = 12
TYPE_UINT32 = 13
TYPE_ENUM = 14
TYPE_SFIXED32 = 15
TYPE_SFIXED64 = 16
TYPE_SINT32 = 17
TYPE_SINT64 = 18
MAX_TYPE = 18

# These are labels in the descriptor. Semantic protobufs only distinguish
# between optional and repeated labels. Required is not enforced by the library
# - it should be done by the user in their Validate() method.
LABEL_OPTIONAL = 1
LABEL_REQUIRED = 2
LABEL_REPEATED = 3
MAX_LABEL = 3

# Semantic Value data store type specifies how they prefer to be encoded. This
# maps to a proto2 primitive field type. When parsing the .proto file we must
# ensure that the semantic value is getting encoded into the correct primitive
# field type.
_SEMANTIC_PRIMITIVE_TO_FIELD_TYPE = dict(
    bytes=TYPE_BYTES,
    string=TYPE_STRING,
    integer=TYPE_INT64,
    unsigned_integer_32=TYPE_UINT32,
    unsigned_integer=TYPE_UINT64,
)

# If True, check that protobuf dependencies are correctly mirrored in
# RDFProtoStruct.rdf_deps field.
CHECK_PROTOBUF_DEPENDENCIES = True


def DefineFromWireFormat(cls, protobuf):
  """Add type info definitions from an existing protobuf.

  We support building this class by copying definitions from an annotated
  protobuf using the semantic protobuf. This is ideal for interoperability
  with other languages and non-semantic protobuf implementations. In that case
  it might be easier to simply annotate the .proto file with the relevant
  semantic information.

  Args:
    cls: The class to add fields descriptors to (i.e. the new semantic class).
    protobuf: A generated proto2 protocol buffer class as produced by the
      standard Google protobuf compiler.

  Raises:
    ProtobufNameMustMatchClassOrParentClassError: if cls and protobuf have
    different names.
  """
  mro_chain = [c.__name__ for c in inspect.getmro(cls)]
  if protobuf.__name__ not in mro_chain and not getattr(
      cls, "allow_custom_class_name", False
  ):
    raise ProtobufNameMustMatchClassOrParentClassError(
        "Can't define RDFProtoStruct class %s from proto %s "
        "(proto name must match one of the classes in the MRO chain: %s)"
        % (cls.__name__, protobuf.__name__, ", ".join(mro_chain))
    )

  cls.recorded_rdf_deps = set()

  # Parse message level options.
  message_options = protobuf.DESCRIPTOR.GetOptions()
  semantic_options = message_options.Extensions[semantic_pb2.semantic]

  # Hack to avoid dependency loop.
  # TODO(amoser): remove this hack
  classes_dict = type_info.TypeInfoObject.classes

  # Support message descriptions
  if semantic_options.description and not cls.__doc__:
    cls.__doc__ = semantic_options.description

  cls.union_field = semantic_options.union_field or None

  # We search through all the field descriptors and build type info
  # descriptors from them.
  for field in protobuf.DESCRIPTOR.fields:
    type_descriptor = None

    # Does this field have semantic options?
    options = field.GetOptions().Extensions[semantic_pb2.sem_type]
    kwargs = dict(
        description=options.description,
        name=field.name,
        friendly_name=options.friendly_name,
        field_number=field.number,
        labels=list(options.label),
    )

    if field.has_default_value:
      kwargs["default"] = field.default_value

    # This field is a non-protobuf semantic value.
    if options.type and field.type != TYPE_MESSAGE:
      cls.recorded_rdf_deps.add(options.type)
      rdf_type = rdfvalue.RDFValue.classes.get(options.type)
      if rdf_type:
        if (
            CHECK_PROTOBUF_DEPENDENCIES
            and rdf_type not in cls.rdf_deps
            and options.type not in cls.rdf_deps
        ):
          raise rdfvalue.InitializeError(
              "%s.%s: field %s is of type %s, "
              "but type is missing from its dependencies list"
              % (cls.__module__, cls.__name__, field.name, options.type)
          )

        # Make sure that the field type is the same as what is required by the
        # semantic type.
        required_field_type = _SEMANTIC_PRIMITIVE_TO_FIELD_TYPE[
            rdf_type.protobuf_type
        ]

        if required_field_type != field.type:
          raise rdfvalue.InitializeError(
              (
                  "%s: .proto file uses incorrect field to store Semantic Value"
                  " %s: Should be %s"
              )
              % (cls.__name__, field.name, rdf_type.protobuf_type)
          )

      type_descriptor = classes_dict["ProtoRDFValue"](
          rdf_type=options.type, **kwargs
      )

    # A semantic protobuf is already a semantic value so it is an error to
    # specify it in two places.
    elif options.type and field.type == TYPE_MESSAGE:
      raise rdfvalue.InitializeError(
          (
              "%s: .proto file specified both Semantic Value type %s and "
              "Semantic protobuf %s"
          )
          % (cls.__name__, options.type, field.message_type.name)
      )

    # Try to figure out what this field actually is from the descriptor.
    elif field.type == TYPE_DOUBLE:
      type_descriptor = classes_dict["ProtoDouble"](**kwargs)

    elif field.type == TYPE_FLOAT:
      type_descriptor = classes_dict["ProtoFloat"](**kwargs)

    elif field.type == TYPE_BOOL:
      type_descriptor = classes_dict["ProtoBoolean"](**kwargs)

    elif field.type == TYPE_STRING:
      type_descriptor = classes_dict["ProtoString"](**kwargs)

    elif field.type == TYPE_BYTES:
      type_descriptor = classes_dict["ProtoBinary"](**kwargs)
      if options.dynamic_type:
        # This may be a dynamic type. In this case the dynamic_type option
        # names a method (which must exist) which should return the class of
        # the embedded semantic value.
        dynamic_cb = getattr(cls, options.dynamic_type, None)
        if dynamic_cb is not None:
          type_descriptor = classes_dict["ProtoDynamicEmbedded"](
              dynamic_cb=dynamic_cb, **kwargs
          )
        else:
          logging.warning(
              "Dynamic type specifies a non existent callback %s",
              options.dynamic_type,
          )

    elif field.type == TYPE_MESSAGE and field.message_type.name == "Any":
      if options.no_dynamic_type_lookup:
        type_descriptor = classes_dict["ProtoAnyValue"](**kwargs)
      else:
        dynamic_cb = getattr(cls, options.dynamic_type, None)
        type_descriptor = classes_dict["ProtoDynamicAnyValueEmbedded"](
            dynamic_cb=dynamic_cb, **kwargs
        )

    elif field.type == TYPE_INT64 or field.type == TYPE_INT32:
      type_descriptor = classes_dict["ProtoSignedInteger"](**kwargs)

    elif field.type == TYPE_UINT32 or field.type == TYPE_UINT64:
      type_descriptor = classes_dict["ProtoUnsignedInteger"](**kwargs)

    # An embedded protocol buffer.
    elif field.type == TYPE_MESSAGE and field.message_type:
      # Refer to another protobuf. Note that the target does not need to be
      # known at this time. It will be resolved using the late binding algorithm
      # when it is known. Therefore this can actually also refer to this current
      # protobuf (i.e. nested proto).
      type_descriptor = classes_dict["ProtoEmbedded"](
          nested=field.message_type.name, **kwargs
      )

      cls.recorded_rdf_deps.add(field.message_type.name)
      if CHECK_PROTOBUF_DEPENDENCIES:
        found = False
        for d in cls.rdf_deps:
          if (
              hasattr(d, "__name__")
              and d.__name__ == field.message_type.name
              or d == field.message_type.name
          ):
            found = True

        if not found:
          raise rdfvalue.InitializeError(
              "%s.%s: TYPE_MESSAGE field %s is %s, "
              "but type is missing from its dependencies list"
              % (
                  cls.__module__,
                  cls.__name__,
                  field.name,
                  field.message_type.name,
              )
          )

      # TODO(user): support late binding here.
      if type_descriptor.type:
        # This traps the following problem:
        # class Certificate(rdf_protodict.RDFValueArray):
        #    protobuf = jobs_pb2.BlobArray
        #

        # A primitive Protobuf definition like:
        # message Certificate {
        #   ....
        # };

        # And a field like:
        # optional Certificate csr = 1 [(sem_type) = {
        #   description: "A Certificate RDFValue with the CSR in it.",
        # }];

        # If we blindly allowed the Certificate RDFValue to be used, the
        # semantic library will end up embedding a BlobArray protobuf, but the
        # primitive library will still use Certificate.

        # The name of the primitive protobuf the semantic type implements.
        semantic_protobuf_primitive = type_descriptor.type.protobuf.__name__

        # This is an error because the primitive library will use the protobuf
        # named in the field, but the semantic library will implement a
        # different protobuf.
        if semantic_protobuf_primitive != field.message_type.name:
          raise rdfvalue.InitializeError(
              (
                  "%s.%s: Conflicting primitive (%s) and semantic protobuf %s "
                  "which implements primitive protobuf (%s)"
              )
              % (
                  cls.__name__,
                  field.name,
                  field.message_type.name,
                  type_descriptor.type.__name__,
                  semantic_protobuf_primitive,
              )
          )

    elif field.enum_type:  # It is an enum.
      # TODO(hanuszczak): Protobuf descriptors use `bytes` objects to represent
      # string values. Hence, we add additional `unicode` calls to convert them.
      # It should be investigated whether this behaviour is needed in Python 3
      # as well.

      enum_desc = field.enum_type
      enum_desc_name = str(enum_desc.name)
      enum_dict = {}
      enum_descriptions = {}
      enum_labels = {}

      for enum_value in enum_desc.values:
        enum_value_name = str(enum_value.name)

        enum_dict[enum_value_name] = enum_value.number
        description = enum_value.GetOptions().Extensions[
            semantic_pb2.description
        ]
        enum_descriptions[enum_value_name] = description
        labels = [
            label
            for label in enum_value.GetOptions().Extensions[semantic_pb2.label]
        ]
        enum_labels[enum_value_name] = labels

      type_descriptor = classes_dict["ProtoEnum"](
          enum_name=enum_desc_name,
          enum=enum_dict,
          enum_descriptions=enum_descriptions,
          enum_labels=enum_labels,
          **kwargs,
      )

      # Attach the enum container to the class for easy reference:
      setattr(cls, enum_desc_name, type_descriptor.enum_container)

    # If we do not recognize the type descriptor we ignore this field.
    if type_descriptor is not None:
      # If the field is repeated, wrap it in a ProtoList.
      if field.label == LABEL_REPEATED:
        options = field.GetOptions().Extensions[semantic_pb2.sem_type]
        type_descriptor = classes_dict["ProtoList"](
            type_descriptor, labels=list(options.label)
        )

      try:
        cls.AddDescriptor(type_descriptor)
      except Exception:
        logging.error("Failed to parse protobuf %s", cls)
        raise

    else:
      logging.error("Unknown field type for %s - Ignoring.", field.name)

  if hasattr(cls, "rdf_deps"):
    leftover_deps = set()
    for d in cls.rdf_deps:
      try:
        leftover_deps.add(d.__name__)
      except AttributeError:
        leftover_deps.add(d)
    for d in cls.recorded_rdf_deps:
      leftover_deps.remove(d)
    if leftover_deps:
      raise rdfvalue.InitializeError(
          "Found superfluous dependencies for %s: %s"
          % (cls.__name__, ",".join(leftover_deps))
      )



#!/usr/bin/env python
# Copyright 2012 Google Inc. All Rights Reserved.
from absl import app

from grr_response_core.lib import type_info
from grr_response_core.lib.rdfvalues import paths as rdf_paths
from grr.test_lib import test_lib


class TypeInfoTest(test_lib.GRRBaseTest):

  def testTypeInfoBoolObjects(self):
    """Test the type info objects behave as expected."""
    a = type_info.Bool()
    self.assertRaises(type_info.TypeValueError, a.Validate, 2)
    self.assertRaises(type_info.TypeValueError, a.Validate, None)
    a.Validate(True)
    # 1 is a valid substitute for True.
    a.Validate(1)

  def testTypeInfoStringObjects(self):
    """Test the type info objects behave as expected."""
    a = type_info.String()
    self.assertRaises(type_info.TypeValueError, a.Validate, 1)
    self.assertRaises(type_info.TypeValueError, a.Validate, None)
    a.Validate("test")
    a.Validate("test")
    a.Validate("/test-Îñ铁网åţî[öñåļ(îžåţîờñ")

  def testTypeInfoNumberObjects(self):
    """Test the type info objects behave as expected."""
    a = type_info.Integer()
    self.assertRaises(type_info.TypeValueError, a.Validate, "1")
    self.assertRaises(type_info.TypeValueError, a.Validate, "hello")
    a.Validate(1231232)
    a.Validate(-2)

  def testTypeInfoListObjects(self):
    """Test List objects."""
    a = type_info.List(type_info.Integer())
    self.assertRaises(type_info.TypeValueError, a.Validate, 1)
    self.assertRaises(type_info.TypeValueError, a.Validate, "test")
    self.assertRaises(type_info.TypeValueError, a.Validate, None)
    self.assertRaises(type_info.TypeValueError, a.Validate, ["test"])
    self.assertRaises(
        type_info.TypeValueError, a.Validate, [rdf_paths.PathSpec()]
    )
    a.Validate([1, 2, 3])

  def testTypeInfoListConvertsObjectsOnValidation(self):
    """Test List objects return validated objects."""

    class TypeInfoFoo(type_info.Integer):

      def Validate(self, value):
        return value * 2

    a = type_info.List(TypeInfoFoo())
    self.assertEqual(a.Validate([1, 2, 3]), [2, 4, 6])

  def testTypeInfoMultiChoiceObjects(self):
    """Test MultiChoice objects."""
    a = type_info.MultiChoice(choices=["a", "b"])
    self.assertRaises(type_info.TypeValueError, a.Validate, "a")
    self.assertRaises(type_info.TypeValueError, a.Validate, ["test"])
    self.assertRaises(type_info.TypeValueError, a.Validate, ["a", "test"])
    self.assertRaises(type_info.TypeValueError, a.Validate, ["a", "a"])
    self.assertRaises(type_info.TypeValueError, a.Validate, None)
    self.assertRaises(type_info.TypeValueError, a.Validate, [1])
    self.assertRaises(type_info.TypeValueError, a.Validate, 1)
    a.Validate(["a"])
    a.Validate(["a", "b"])

    with self.assertRaises(type_info.TypeValueError):
      type_info.MultiChoice(choices=[1, 2])

    a = type_info.MultiChoice(choices=[1, 2], validator=type_info.Integer())
    self.assertRaises(type_info.TypeValueError, a.Validate, "a")
    self.assertRaises(type_info.TypeValueError, a.Validate, ["test"])
    a.Validate([2])
    a.Validate([1, 2])

  def testStructDictType(self):
    """Test RDFStructDictType type."""
    a = type_info.RDFStructDictType(rdfclass=rdf_paths.PathSpec)
    self.assertRaises(type_info.TypeValueError, a.Validate, 2)
    self.assertRaises(type_info.TypeValueError, a.Validate, "ab")
    self.assertRaises(type_info.TypeValueError, a.Validate, [1, 2])

    # None values are left as is.
    self.assertEqual(None, a.Validate(None))

    # Check that validation raises when unknown attributes are passed.
    self.assertRaises(type_info.TypeValueError, a.Validate, dict(foo="bar"))

    v = a.Validate(dict(path="blah", pathtype="TSK"))
    self.assertIsInstance(v, rdf_paths.PathSpec)
    self.assertEqual(v.path, "blah")
    self.assertEqual(v.pathtype, "TSK")

  def testTypeDescriptorSet(self):

    type_infos = [
        type_info.String(name="output", default="analysis/{p}/{u}-{t}"),
        type_info.String(
            description="Profile to use.", name="profile", default=""
        ),
        type_info.String(
            description="A comma separated list of plugins.",
            name="plugins",
            default="",
        ),
    ]

    info = type_info.TypeDescriptorSet(
        type_infos[0],
        type_infos[1],
        type_infos[2],
    )

    new_info = type_info.TypeDescriptorSet(
        type_infos[0],
    )

    updated_info = new_info + type_info.TypeDescriptorSet(
        type_infos[1],
    )

    updated_info += type_info.TypeDescriptorSet(
        type_infos[2],
    )

    self.assertEqual(info.descriptor_map, updated_info.descriptor_map)
    self.assertCountEqual(info.descriptors, updated_info.descriptors)

    self.assertIn(type_infos[1], updated_info.descriptors)
    self.assertIn("plugins", updated_info)

    removed_info = updated_info.Remove("plugins")

    self.assertIn(type_infos[1], updated_info.descriptors)
    self.assertIn("plugins", updated_info)

    self.assertNotIn(type_infos[2], removed_info.descriptors)
    self.assertNotIn("plugins", removed_info)


def main(args):
  test_lib.main(args)


if __name__ == "__main__":
  app.run(main)



#!/usr/bin/env python
"""Typing information for flow arguments.

This contains objects that are used to provide type annotations for flow
parameters. These annotations are used to assist in rendering the UI for
starting flows and for validating arguments.
"""

from collections.abc import Iterable
import logging
from typing import Optional

from grr_response_core.lib import rdfvalue
from grr_response_core.lib import serialization
from grr_response_core.lib.registry import MetaclassRegistry
from grr_response_core.lib.util import precondition


class Error(Exception):
  """Base error class."""


# TODO(hanuszczak): Consider getting rid of this class. Standard `ValueError`
# and `TypeError` should be used instead.
class TypeValueError(Error, ValueError):
  """Value is not valid."""


class UnknownArg(TypeValueError):
  """Raised for unknown flow args."""


class TypeInfoObject(metaclass=MetaclassRegistry):
  """Definition of the interface for flow arg typing information."""

  # The delegate type this TypeInfoObject manages.
  _type = None

  def __init__(
      self,
      name="",
      default=None,
      description="",
      friendly_name="",
      hidden=False,
  ):
    """Build a TypeInfo type descriptor.

    Args:
      name: The name of the parameter that this Type info corresponds to.
      default: The default value that should be specified if the parameter was
        not set.
      description: A string describing this flow argument.
      friendly_name: A human readable name which may be provided.
      hidden: Should the argument be hidden from the UI.
    """
    self.name = name
    self.section = name.split(".")[0]
    self.default = default
    self.description = description
    self.hidden = hidden
    if not friendly_name:
      friendly_name = name.replace("_", " ").capitalize()

    self.friendly_name = friendly_name

    # It is generally impossible to check the default value here
    # because this happens before any configuration is loaded (i.e. at
    # import time). Hence default values which depend on the config
    # system can't be tested. We just assume that the default value is
    # sensible since its hard coded in the code.

  def GetType(self):
    """Returns the type class described by this type info."""
    return self._type

  def GetDefault(self):
    """Return the default value for this TypeInfoObject."""
    return self.default

  def Validate(self, value):
    """Confirm that the value is valid for this type.

    Args:
      value: The value being used to initialize the flow.

    Raises:
      TypeValueError: On value not conforming to type.

    Returns:
      A potentially modified value if we can use the provided value to construct
      a valid input.
    """
    return value

  def FromString(self, string):
    return string

  def ToString(self, value):
    return str(value)

  def Help(self):
    """Returns a helpful string describing this type info."""
    return "%s\n   Description: %s\n   Default: %s" % (
        self.name,
        self.description,
        self.GetDefault(),
    )


class RDFValueType(TypeInfoObject):
  """An arg which must be an RDFValue."""

  rdfclass = rdfvalue.RDFValue

  def __init__(self, rdfclass=None, **kwargs):
    """An arg which must be an RDFValue.

    Args:
      rdfclass: The RDFValue class that this arg must be.
      **kwargs: Passthrough to base class.
    """
    super().__init__(**kwargs)
    self._type = self.rdfclass = rdfclass

  def Validate(self, value):
    """Validate an RDFValue instance.

    Args:
      value: An RDFValue instance or something which may be used to instantiate
        the correct instance.

    Raises:
      TypeValueError: If the value is not a valid RDFValue instance or the
        required type.

    Returns:
      A Valid RDFValue instance.
    """
    # Allow None as a default.
    if value is None:
      return

    if not isinstance(value, self.rdfclass):
      # Try to coerce the type to the correct rdf_class.
      try:
        return self.rdfclass(value)
      except rdfvalue.InitializeError as e:
        raise TypeValueError(
            "Value for arg %s should be an %s"
            % (self.name, self.rdfclass.__name__)
        ) from e

    return value

  def FromString(self, string):
    return serialization.FromHumanReadable(self.rdfclass, string)


# This file can't depend on rdf_structs (cyclic dependency), so
# args/return values of type EnumContainer are not type annotated.
class RDFEnumType(TypeInfoObject):
  """An arg which must be an stringified enum value."""

  def __init__(self, enum_container, **kwargs):
    super().__init__(**kwargs)
    self._enum_container = enum_container

  def Value(self, value: Optional[str]):
    if value is None:
      return

    if isinstance(value, str):
      return self.FromString(value)

    raise ValueError("Invalid value {value} for RDFEnumType.")

  def FromString(self, string: str):
    return self._enum_container.FromString(string)


class RDFStructDictType(TypeInfoObject):
  """An arg which must be a dict that maps into an RDFStruct."""

  rdfclass = rdfvalue.RDFValue

  def __init__(self, rdfclass=None, **kwargs):
    """An arg which must be an RDFStruct.

    Args:
      rdfclass: The RDFStruct subclass that this arg must be.
      **kwargs: Passthrough to base class.
    """
    super().__init__(**kwargs)
    self._type = self.rdfclass = rdfclass

  def Validate(self, value):
    """Validate the value.

    Args:
      value: Value is expected to be a dict-like object that a given RDFStruct
        can be initialized from.

    Raises:
      TypeValueError: If the value is not a valid dict-like object that a given
        RDFStruct can be initialized from.

    Returns:
      A valid instance of self.rdfclass or None.
    """
    if value is None:
      return None

    if not isinstance(value, self.rdfclass):
      # Try to coerce the type to the correct rdf_class.
      try:
        r = self.rdfclass()
        r.FromDict(value)
        return r
      except (AttributeError, TypeError, rdfvalue.InitializeError) as e:
        # AttributeError is raised if value contains items that don't
        # belong to the given rdfstruct.
        # TypeError will be raised if value is not a dict-like object.
        raise TypeValueError(
            "Value for arg %s should be an %s"
            % (self.name, self.rdfclass.__name__)
        ) from e

    return value

  def FromString(self, string):
    return self.rdfclass.FromSerializedBytes(string)


class TypeDescriptorSet(Iterable[TypeInfoObject]):
  """This is a collection of type descriptors.

  This collections is effectively immutable. Add/Remove operations create new
  set instead of modifying existing one.
  """

  def __init__(self, *descriptors):
    self.descriptors = list(descriptors)
    self.descriptor_names = [x.name for x in descriptors]
    self.descriptor_map = dict([(desc.name, desc) for desc in descriptors])

  def __getitem__(self, item):
    return self.descriptor_map[item]

  def __contains__(self, item):
    return item in self.descriptor_map

  def get(self, item, default=None):  # pylint: disable=g-bad-name
    return self.descriptor_map.get(item, default)

  def __iter__(self):
    return iter(self.descriptors)

  def __str__(self):
    result = "\n ".join(
        ["%s: %s" % (x.name, x.description) for x in self.descriptors]
    )

    return "<TypeDescriptorSet for %s>\n %s\n</TypeDescriptorSet>\n" % (
        self.__class__.__name__,
        result,
    )

  def __add__(self, other):
    return self.Add(other)

  def __radd__(self, other):
    return self.Add(other)

  def Add(self, other):
    """Returns a copy of this set with a new element added."""
    new_descriptors = []
    for desc in self.descriptors + other.descriptors:
      if desc not in new_descriptors:
        new_descriptors.append(desc)

    return TypeDescriptorSet(*new_descriptors)

  def Append(self, desc):
    """Append the descriptor to this set."""
    if desc not in self.descriptors:
      self.descriptors.append(desc)
      self.descriptor_map[desc.name] = desc
      self.descriptor_names.append(desc.name)

  def HasDescriptor(self, descriptor_name):
    """Checks whether this set has an element with the given name."""
    return descriptor_name in self.descriptor_map

  def Remove(self, *descriptor_names):
    """Returns a copy of this set without elements with given names."""
    new_descriptor_map = self.descriptor_map.copy()
    for name in descriptor_names:
      new_descriptor_map.pop(name, None)

    new_descriptors = [
        desc for desc in self.descriptors if desc in new_descriptor_map.values()
    ]
    return TypeDescriptorSet(*new_descriptors)

  def ParseArgs(self, args):
    """Parse and validate the args.

    Note we pop all the args we consume here - so if there are any args we dont
    know about, args will not be an empty dict after this. This allows the same
    args to be parsed by several TypeDescriptorSets.

    Args:
      args: A dictionary of arguments that this TypeDescriptorSet might use. If
        this dict does not have a required parameter, we still yield its default
        value.

    Yields:
      A (name, value) tuple of the parsed args.
    """
    for descriptor in self:
      # Get the value from the kwargs or, if not specified, the default.
      value = args.pop(descriptor.name, None)

      if value is None:
        # No need to validate the default value.
        value = descriptor.default
      else:
        try:
          # Validate this value - this should raise if the value provided is not
          # acceptable to the type descriptor.
          value = descriptor.Validate(value)
        except Exception:
          logging.error("Invalid value %s for arg %s", value, descriptor.name)
          raise

      yield descriptor.name, value


class Bool(TypeInfoObject):
  """A True or False value."""

  _type = bool

  def Validate(self, value):
    if value not in [True, False]:
      raise TypeValueError("Value must be True or False")

    return value

  def FromString(self, string):
    """Parse a bool from a string."""
    if string.lower() in ("false", "no", "n"):
      return False

    if string.lower() in ("true", "yes", "y"):
      return True

    raise TypeValueError("%s is not recognized as a boolean value." % string)


class List(TypeInfoObject):
  """A list type. Turns another type into a list of those types."""

  _type = list

  def __init__(self, validator=None, **kwargs):
    self.validator = validator
    super().__init__(**kwargs)

  def Validate(self, value):
    """Validate a potential list."""
    if isinstance(value, str):
      raise TypeValueError("Value must be an iterable not a string.")

    elif not isinstance(value, (list, tuple)):
      raise TypeValueError("%r not a valid List" % value)

    # Validate each value in the list validates against our type.
    return [self.validator.Validate(val) for val in value]

  def FromString(self, string):
    result = []
    if string:
      for x in string.split(","):
        x = x.strip()
        result.append(self.validator.FromString(x))

    return result

  def ToString(self, value):
    return ",".join([self.validator.ToString(x) for x in value])


class String(TypeInfoObject):
  """A String type."""

  _type = str

  def __init__(self, default: str = "", **kwargs):
    precondition.AssertType(default, str)
    super().__init__(default=default, **kwargs)

  def Validate(self, value: str) -> str:
    if not isinstance(value, str):
      raise TypeValueError("'{}' is not a valid string".format(value))

    return value

  def FromString(self, string: str) -> str:
    precondition.AssertType(string, str)
    return string

  def ToString(self, value: str) -> str:
    precondition.AssertType(value, str)
    return value


class Bytes(TypeInfoObject):
  """A Bytes type."""

  _type = bytes

  def __init__(self, default: bytes = b"", **kwargs):
    precondition.AssertType(default, bytes)
    super().__init__(default=default, **kwargs)

  def Validate(self, value: bytes) -> bytes:
    if not isinstance(value, bytes):
      raise TypeValueError("%s not a valid string" % value)

    return value

  def FromString(self, string: str) -> bytes:
    precondition.AssertType(string, str)
    return string.encode("utf-8")

  def ToString(self, value: bytes) -> str:
    precondition.AssertType(value, bytes)
    return value.decode("utf-8")


class Integer(TypeInfoObject):
  """An Integer number type."""

  _type = int

  def Validate(self, value):
    if value is None:
      value = 0

    if not isinstance(value, int):
      raise TypeValueError("Invalid value %s for Integer" % value)

    return value

  def FromString(self, string):
    try:
      return int(string)
    except ValueError:
      raise TypeValueError("Invalid value %s for Integer" % string)


class Float(Integer):
  """Type info describing a float."""

  _type = float

  def Validate(self, value):
    try:
      value = float(value)
    except (ValueError, TypeError):
      raise TypeValueError("Invalid value %s for Float" % value)

    return value

  def FromString(self, string):
    try:
      return float(string)
    except (ValueError, TypeError):
      raise TypeValueError("Invalid value %s for Float" % string)


class Choice(TypeInfoObject):
  """A choice from a set of allowed values."""

  def __init__(self, choices=None, validator=None, **kwargs):
    self.choices = choices
    self.validator = validator or String()
    super().__init__(**kwargs)

  def Validate(self, value):
    self.validator.Validate(value)

    if value not in self.choices:
      raise TypeValueError("%s not a valid instance string." % value)

    return value


class MultiChoice(TypeInfoObject):
  """Choose a list of values from a set of allowed values."""

  def __init__(self, choices=None, validator=None, **kwargs):
    """Create a multichoice object and validate choices.

    Args:
      choices: list of available choices
      validator: validator to use for each of the list *items* the validator for
        the top level is a list.
      **kwargs: passed through to parent class.
    """
    self.choices = choices
    subvalidator = validator or String()
    self.validator = List(validator=subvalidator)

    # Check the choices match the validator
    for choice in self.choices:
      subvalidator.Validate(choice)
    super().__init__(**kwargs)

  def Validate(self, values):
    self.validator.Validate(values)

    for value in values:
      if value not in self.choices:
        raise TypeValueError("%s not a valid instance string." % value)
    if len(values) != len(set(values)):
      raise TypeValueError("Duplicate choice in: %s." % values)
    return values



#!/usr/bin/env python
"""Setup configuration for the python grr modules."""

import configparser
import itertools
import os
import shutil
import subprocess
import sys

from setuptools import Extension
from setuptools import find_packages
from setuptools import setup
from setuptools.command.develop import develop
from setuptools.command.install import install
from setuptools.command.sdist import sdist

THIS_DIRECTORY = os.path.dirname(os.path.realpath(__file__))
os.chdir(THIS_DIRECTORY)


def find_data_files(source, ignore_dirs=None):
  ignore_dirs = ignore_dirs or []
  result = []
  for directory, dirnames, files in os.walk(source):
    dirnames[:] = [d for d in dirnames if d not in ignore_dirs]

    files = [os.path.join(directory, x) for x in files]
    result.append((directory, files))

  return result


def sync_artifacts():
  """Sync the artifact repo with upstream for distribution."""

  subprocess.check_call([sys.executable, "makefile.py"],
                        cwd="grr_response_core/artifacts")


def get_config():
  """Get relative path to version.ini file and the INI parser with its data."""
  rel_ini_path = "version.ini"
  ini_path = os.path.join(THIS_DIRECTORY, rel_ini_path)
  if not os.path.exists(ini_path):
    rel_ini_path = os.path.join("..", "..", "version.ini")
    ini_path = os.path.join(THIS_DIRECTORY, rel_ini_path)
    if not os.path.exists(ini_path):
      raise RuntimeError("Couldn't find version.ini")

  config = configparser.ConfigParser()
  config.read(ini_path)
  return rel_ini_path, config


REL_INI_PATH, VERSION = get_config()


class Develop(develop):

  def run(self):
    sync_artifacts()

    develop.run(self)


class Sdist(sdist):
  """Build sdist."""

  user_options = sdist.user_options + [
      ("no-sync-artifacts", None,
       "Don't sync the artifact repo. This is unnecessary for "
       "clients and old client build OSes can't make the SSL connection."),
  ]

  def initialize_options(self):
    self.no_sync_artifacts = None

    sdist.initialize_options(self)

  def run(self):
    if not self.no_sync_artifacts:
      sync_artifacts()

    sdist.run(self)

  def make_release_tree(self, base_dir, files):
    sdist.make_release_tree(self, base_dir, files)
    sdist_version_ini = os.path.join(base_dir, "version.ini")
    if os.path.exists(sdist_version_ini):
      os.unlink(sdist_version_ini)
    shutil.copy(
        os.path.join(THIS_DIRECTORY, "../../version.ini"), sdist_version_ini)


data_files = list(
    itertools.chain(
        find_data_files("executables"),
        find_data_files("install_data"),
        find_data_files("scripts"),
        find_data_files("grr_response_core/artifacts"),
        [REL_INI_PATH],
    )
)

setup_args = dict(
    name="grr-response-core",
    version=VERSION.get("Version", "packageversion"),
    description="GRR Rapid Response",
    license="Apache License, Version 2.0",
    url="https://github.com/google/grr",
    maintainer="GRR Development Team",
    maintainer_email="grr-dev@googlegroups.com",
    python_requires=">=3.6",
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    ext_modules=[
        Extension(
            name="grr_response_core._semantic",
            sources=["accelerated/accelerated.c"],
        )
    ],
    cmdclass={
        "develop": Develop,
        "install": install,
        "sdist": Sdist,
    },
    install_requires=[
        "cryptography==3.4.8",
        "distro==1.7.0",
        "fleetspeak==0.1.12",
        "grr-response-proto==%s" % VERSION.get("Version", "packagedepends"),
        "ipaddr==2.2.0",
        "pexpect==4.8.0",
        "pip>=21.0.1",
        "psutil==5.8.0",
        "python-crontab==2.5.1",
        "python-dateutil==2.8.2",
        "pytz==2022.7.1",
        "PyYAML==6.0.1",
        "requests==2.25.1",
        "yara-python==4.5.1",
    ],
    # Data files used by GRR. Access these via the config_lib "resource" filter.
    data_files=data_files,
)

setup(**setup_args)



#!/usr/bin/env python
"""Cloud-related rdfvalues."""

from grr_response_core.lib.rdfvalues import protodict as rdf_protodict
from grr_response_core.lib.rdfvalues import structs as rdf_structs
from grr_response_proto import flows_pb2
from grr_response_proto import jobs_pb2

AMAZON_URL_BASE = "http://169.254.169.254/latest/meta-data"
AMAZON_BIOS_REGEX = ".*amazon"
AMAZON_SERVICE_REGEX = "SERVICE_NAME: AWSLiteAgent"
# Using the ip and not metadata.google.internal to avoid issues on endpoints
# with misconfigured DNS resolvers.
GOOGLE_URL_BASE = "http://169.254.169.254/computeMetadata/v1"
GOOGLE_BIOS_REGEX = "Google"
GOOGLE_SERVICE_REGEX = "SERVICE_NAME: GCEAgent"


class CloudMetadataRequest(rdf_structs.RDFProtoStruct):
  """RDF wrapper for `CloudMetadataRequest` message."""

  protobuf = flows_pb2.CloudMetadataRequest
  rdf_deps = [
      rdf_protodict.Dict,
  ]

  @classmethod
  def ForAmazon(cls, *args, **kwargs) -> "CloudMetadataRequest":
    return cls(
        *args,
        **kwargs,
        bios_version_regex=AMAZON_BIOS_REGEX,
        service_name_regex=AMAZON_SERVICE_REGEX,
        instance_type=CloudInstance.InstanceType.AMAZON,
        timeout=1.0,
    )

  @classmethod
  def ForGoogle(cls, *args, **kwargs) -> "CloudMetadataRequest":
    return cls(
        *args,
        **kwargs,
        bios_version_regex=GOOGLE_BIOS_REGEX,
        service_name_regex=GOOGLE_SERVICE_REGEX,
        headers={"Metadata-Flavor": "Google"},
        instance_type=CloudInstance.InstanceType.GOOGLE,
        timeout=1.0,
    )


class CloudMetadataRequests(rdf_structs.RDFProtoStruct):
  protobuf = flows_pb2.CloudMetadataRequests
  rdf_deps = [
      CloudMetadataRequest,
  ]


class CloudMetadataResponse(rdf_structs.RDFProtoStruct):
  protobuf = flows_pb2.CloudMetadataResponse


class CloudMetadataResponses(rdf_structs.RDFProtoStruct):
  protobuf = flows_pb2.CloudMetadataResponses
  rdf_deps = [
      CloudMetadataResponse,
  ]


class GoogleCloudInstance(rdf_structs.RDFProtoStruct):
  protobuf = jobs_pb2.GoogleCloudInstance


class AmazonCloudInstance(rdf_structs.RDFProtoStruct):
  protobuf = jobs_pb2.AmazonCloudInstance


class CloudInstance(rdf_structs.RDFProtoStruct):
  protobuf = jobs_pb2.CloudInstance
  rdf_deps = [
      AmazonCloudInstance,
      GoogleCloudInstance,
  ]


# TODO: Return a proto instead of an RDFValue.
def BuildCloudMetadataRequests():
  """Build the standard set of cloud metadata to collect during interrogate.

  The `label` field is used to identify the metadata in the response, and should
  match a valid field of the corresponding <Platform>CloudInstance proto.
  Example: A request for instance_type=CloudInstance.InstanceType.GOOGLE with
          `label="instance_id"` -> `GoogleCloudInstance.instance_id`

  Returns:
    A CloudMetadataRequests proto with the standard set of interrogate requests.
  """
  return CloudMetadataRequests(
      requests=[
          CloudMetadataRequest.ForAmazon(
              url="/".join((AMAZON_URL_BASE, "instance-id")),
              label="instance_id",
          ),
          CloudMetadataRequest.ForAmazon(
              url="/".join((AMAZON_URL_BASE, "ami-id")),
              label="ami_id",
          ),
          CloudMetadataRequest.ForAmazon(
              url="/".join((AMAZON_URL_BASE, "hostname")),
              label="hostname",
          ),
          CloudMetadataRequest.ForAmazon(
              url="/".join((AMAZON_URL_BASE, "public-hostname")),
              label="public_hostname",
              # Per AWS documentation, `public-hostname` is available only on
              # hosts with public IPv4 addresses and `enableDnsHostnames` option
              # set. Otherwise, attempts to query this property will result in
              # 404 HTTP status.
              #
              # https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instancedata-data-categories.html
              ignore_http_errors=True,
          ),
          CloudMetadataRequest.ForAmazon(
              url="/".join((AMAZON_URL_BASE, "instance-type")),
              label="instance_type",
          ),
          CloudMetadataRequest.ForGoogle(
              url="/".join((GOOGLE_URL_BASE, "instance/id")),
              label="instance_id",
          ),
          CloudMetadataRequest.ForGoogle(
              url="/".join((GOOGLE_URL_BASE, "instance/zone")),
              label="zone",
          ),
          CloudMetadataRequest.ForGoogle(
              url="/".join((GOOGLE_URL_BASE, "project/project-id")),
              label="project_id",
          ),
          CloudMetadataRequest.ForGoogle(
              url="/".join((GOOGLE_URL_BASE, "instance/hostname")),
              label="hostname",
          ),
          CloudMetadataRequest.ForGoogle(
              url="/".join((GOOGLE_URL_BASE, "instance/machine-type")),
              label="machine_type",
          ),
      ],
  )



#!/usr/bin/env python
"""Implementations of RDFValues used in GRR config options definitions."""

from grr_response_core.lib.rdfvalues import structs as rdf_structs
from grr_response_proto import config_pb2


class AdminUIClientWarningRule(rdf_structs.RDFProtoStruct):
  protobuf = config_pb2.AdminUIClientWarningRule


class AdminUIClientWarningsConfigOption(rdf_structs.RDFProtoStruct):
  protobuf = config_pb2.AdminUIClientWarningsConfigOption
  rdf_deps = [AdminUIClientWarningRule]


class AdminUIHuntConfig(rdf_structs.RDFProtoStruct):
  protobuf = config_pb2.AdminUIHuntConfig



#!/usr/bin/env python
"""Abstracts encryption and authentication."""

import logging
import struct

from grr_response_core.lib.rdfvalues import crypto as rdf_crypto
from grr_response_core.lib.rdfvalues import flows as rdf_flows
from grr_response_core.stats import metrics


# Although these metrics are never queried on the client, removing them from the
# client code seems not worth the effort.
GRR_DECODING_ERROR = metrics.Counter("grr_decoding_error")
GRR_DECRYPTION_ERROR = metrics.Counter("grr_decryption_error")
GRR_LEGACY_CLIENT_DECRYPTION_ERROR = metrics.Counter(
    "grr_legacy_client_decryption_error"
)
GRR_RSA_OPERATIONS = metrics.Counter("grr_rsa_operations")


class Error(Exception):
  """Base class for all exceptions in this module."""


class DecodingError(Error):
  """Raised when the message failed to decrypt or decompress."""

  @GRR_DECODING_ERROR.Counted()
  def __init__(self, message):
    super().__init__(message)


class DecryptionError(DecodingError):
  """Raised when the message can not be decrypted properly."""

  @GRR_DECRYPTION_ERROR.Counted()
  def __init__(self, message):
    super().__init__(message)


class LegacyClientDecryptionError(DecryptionError):
  """Raised when old clients' messages cannot be decrypted."""

  @GRR_LEGACY_CLIENT_DECRYPTION_ERROR.Counted()
  def __init__(self, message):
    super().__init__(message)


class Cipher:
  """Holds keying information."""

  cipher_name = "aes_128_cbc"
  key_size = 128
  iv_size = 128

  # These fields get filled in by the constructor
  private_key = None
  cipher = None
  cipher_metadata = None
  encrypted_cipher = None
  encrypted_cipher_metadata = None

  def __init__(self, source, private_key, remote_public_key):
    self.private_key = private_key

    # The CipherProperties() protocol buffer specifying the session keys, that
    # we send to the other end point. It will be encrypted using the RSA private
    # key.
    self.cipher = rdf_flows.CipherProperties(
        name=self.cipher_name,
        key=rdf_crypto.EncryptionKey.GenerateKey(length=self.key_size),
        metadata_iv=rdf_crypto.EncryptionKey.GenerateKey(length=self.key_size),
        hmac_key=rdf_crypto.EncryptionKey.GenerateKey(length=self.key_size),
        hmac_type="FULL_HMAC",
    )

    serialized_cipher = self.cipher.SerializeToBytes()

    self.cipher_metadata = rdf_flows.CipherMetadata(source=source)

    # Sign this cipher.
    self.cipher_metadata.signature = self.private_key.Sign(serialized_cipher)

    # Now encrypt the cipher.
    GRR_RSA_OPERATIONS.Increment()
    self.encrypted_cipher = remote_public_key.Encrypt(serialized_cipher)

    # Encrypt the metadata block symmetrically.
    _, self.encrypted_cipher_metadata = self.Encrypt(
        self.cipher_metadata.SerializeToBytes(), self.cipher.metadata_iv
    )

  def Encrypt(self, data, iv=None):
    """Symmetrically encrypt the data using the optional iv."""
    if iv is None:
      iv = rdf_crypto.EncryptionKey.GenerateKey(length=128)
    cipher = rdf_crypto.AES128CBCCipher(self.cipher.key, iv)
    return iv, cipher.Encrypt(data)

  def Decrypt(self, data, iv):
    """Symmetrically decrypt the data."""
    key = rdf_crypto.EncryptionKey(self.cipher.key)
    iv = rdf_crypto.EncryptionKey(iv)
    return rdf_crypto.AES128CBCCipher(key, iv).Decrypt(data)

  @property
  def hmac_type(self):
    return self.cipher.hmac_type

  def HMAC(self, *data):
    return rdf_crypto.HMAC(self.cipher.hmac_key).HMAC(b"".join(data))


class ReceivedCipher(Cipher):
  """A cipher which we received from our peer."""

  # pylint: disable=super-init-not-called
  def __init__(
      self, response_comms: rdf_flows.ClientCommunication, private_key
  ):
    self.private_key = private_key
    self.response_comms = response_comms

    if response_comms.api_version not in [3]:
      raise DecryptionError(
          "Unsupported api version: %s, expected 3."
          % response_comms.api_version
      )

    if not response_comms.encrypted_cipher:
      logging.error(
          "No encrypted_cipher. orig_request=%s",
          str(response_comms.orig_request)[:1000],
      )
      # The message is not encrypted. We do not allow unencrypted
      # messages:
      raise DecryptionError("Message is not encrypted.")

    try:
      # The encrypted_cipher contains the session key, iv and hmac_key.
      self.serialized_cipher = private_key.Decrypt(
          response_comms.encrypted_cipher
      )

      # If we get here we have the session keys.
      self.cipher = rdf_flows.CipherProperties.FromSerializedBytes(
          self.serialized_cipher
      )

      # Check the key lengths.
      if (
          len(self.cipher.key) * 8 != self.key_size
          or len(self.cipher.metadata_iv) * 8 != self.iv_size
          or len(self.cipher.hmac_key) * 8 != self.key_size
      ):
        raise DecryptionError("Invalid cipher.")

      self.VerifyHMAC()

      # Cipher_metadata contains information about the cipher - It is encrypted
      # using the symmetric session key. It contains the RSA signature of the
      # digest of the serialized CipherProperties(). It is stored inside the
      # encrypted payload.
      serialized_metadata = self.Decrypt(
          response_comms.encrypted_cipher_metadata, self.cipher.metadata_iv
      )
      self.cipher_metadata = rdf_flows.CipherMetadata.FromSerializedBytes(
          serialized_metadata
      )

    except (rdf_crypto.InvalidSignature, rdf_crypto.CipherError) as e:
      if "Ciphertext length must be equal to key size" in str(e):
        raise LegacyClientDecryptionError(e)
      else:
        raise DecryptionError(e)

  def GetSource(self):
    return self.cipher_metadata.source

  def VerifyReceivedHMAC(self, comms):
    """Verifies a received HMAC.

    This method raises a DecryptionError if the received HMAC does not
    verify. If the HMAC verifies correctly, True is returned.

    Args:
      comms: The comms RdfValue to verify.

    Raises:
      DecryptionError: The HMAC did not verify.

    Returns:
      True
    """
    return self._VerifyHMAC(comms)

  def VerifyHMAC(self):
    """Verifies the HMAC of self.response.comms.

    This method raises a DecryptionError if the received HMAC does not
    verify. If the HMAC verifies correctly, True is returned.

    Raises:
      DecryptionError: The HMAC did not verify.

    Returns:
      True
    """
    return self._VerifyHMAC(self.response_comms)

  def _VerifyHMAC(self, comms=None):
    """Verifies the HMAC.

    This method raises a DecryptionError if the received HMAC does not
    verify. If the HMAC verifies correctly, True is returned.

    Args:
      comms: The comms RdfValue to verify.

    Raises:
      DecryptionError: The HMAC did not verify.

    Returns:
      True
    """
    # Check the encrypted message integrity using HMAC.
    if self.hmac_type == "SIMPLE_HMAC":
      msg = comms.encrypted
      digest = comms.hmac
    elif self.hmac_type == "FULL_HMAC":
      msg = b"".join([
          comms.encrypted,
          comms.encrypted_cipher,
          comms.encrypted_cipher_metadata,
          comms.packet_iv.SerializeToBytes(),
          struct.pack("<I", comms.api_version),
      ])
      digest = comms.full_hmac
    else:
      raise DecryptionError("HMAC type no supported.")

    try:
      rdf_crypto.HMAC(self.cipher.hmac_key).Verify(msg, digest)
    except rdf_crypto.VerificationError as e:
      raise DecryptionError("HMAC verification failed: %s" % e)

    return True

  def VerifyCipherSignature(self, remote_public_key):
    """Verifies the signature on the encrypted cipher block.

    This method returns True if the signature verifies correctly with
    the key given.

    Args:
      remote_public_key: The remote public key.

    Returns:
      None
    Raises:
      rdf_crypto.VerificationError: A signature and a key were both given but
                                    verification fails.
    """
    if self.cipher_metadata.signature and remote_public_key:
      GRR_RSA_OPERATIONS.Increment()
      remote_public_key.Verify(
          self.serialized_cipher, self.cipher_metadata.signature
      )
      return True



#!/usr/bin/env python
"""This module tests the RDFValue implementation for performance."""

from absl import app

from grr_response_core.lib import type_info
from grr_response_core.lib.rdfvalues import client as rdf_client
from grr_response_core.lib.rdfvalues import structs as rdf_structs
from grr_response_proto import jobs_pb2
from grr_response_proto import knowledge_base_pb2
from grr.test_lib import benchmark_test_lib
from grr.test_lib import test_lib


class StructGrrMessage(rdf_structs.RDFProtoStruct):
  """A serialization agnostic GrrMessage."""

  type_description = type_info.TypeDescriptorSet(
      rdf_structs.ProtoString(
          name="session_id",
          field_number=1,
          description="Every Flow has a unique session id.",
      ),
      rdf_structs.ProtoUnsignedInteger(
          name="request_id",
          field_number=2,
          description="This message is in response to this request number",
      ),
      rdf_structs.ProtoUnsignedInteger(
          name="response_id",
          field_number=3,
          description="Responses for each request.",
      ),
      rdf_structs.ProtoString(
          name="name",
          field_number=4,
          description=(
              "This is the name of the client action that will be "
              "executed. It is set by the flow and is executed by "
              "the client."
          ),
      ),
      rdf_structs.ProtoBinary(
          name="args",
          field_number=5,
          description="This field contains an encoded rdfvalue.",
      ),
      rdf_structs.ProtoString(
          name="source",
          field_number=6,
          description=(
              "Client name where the message came from (This is "
              "copied from the MessageList)"
          ),
      ),
  )


class FastGrrMessageList(rdf_structs.RDFProtoStruct):
  """A Faster implementation of GrrMessageList."""

  type_description = type_info.TypeDescriptorSet(
      rdf_structs.ProtoList(
          rdf_structs.ProtoEmbedded(
              name="job", field_number=1, nested=StructGrrMessage
          )
      )
  )


class RDFValueBenchmark(benchmark_test_lib.AverageMicroBenchmarks):
  """Microbenchmark tests for RDFProtos."""

  REPEATS = 1000
  units = "us"

  USER_ACCOUNT = dict(
      username="user",
      full_name="John Smith",
      last_logon=10000,
      userdomain="Some domain name",
      homedir="/home/user",
      sid="some sid",
  )

  def testObjectCreation(self):
    """Compare the speed of object creation to raw protobufs."""
    test_proto = knowledge_base_pb2.User(**self.USER_ACCOUNT)
    test_proto = test_proto.SerializeToString()

    def RDFStructCreateAndSerialize():
      s = rdf_client.User(**self.USER_ACCOUNT)
      s.SerializeToBytes()

    def RDFStructCreateAndSerializeSetValue():
      s = rdf_client.User()
      for k, v in self.USER_ACCOUNT.items():
        setattr(s, k, v)

      s.SerializeToBytes()

    def RDFStructCreateAndSerializeFromProto():
      s = rdf_client.User.FromSerializedBytes(test_proto)
      s.SerializeToBytes()

    def ProtoCreateAndSerialize():
      s = knowledge_base_pb2.User(**self.USER_ACCOUNT)
      s.SerializeToString()

    def ProtoCreateAndSerializeSetValue():
      s = knowledge_base_pb2.User()
      for k, v in self.USER_ACCOUNT.items():
        setattr(s, k, v)

      s.SerializeToString()

    def ProtoCreateAndSerializeFromProto():
      s = knowledge_base_pb2.User()
      s.ParseFromString(test_proto)
      self.assertEqual(s.SerializeToString(), test_proto)

    self.TimeIt(
        RDFStructCreateAndSerialize,
        "SProto Create from keywords and serialize.",
    )

    self.TimeIt(
        RDFStructCreateAndSerializeSetValue, "SProto Create, Set And Serialize"
    )

    self.TimeIt(
        RDFStructCreateAndSerializeFromProto,
        "SProto from serialized and serialize.",
    )

    self.TimeIt(
        ProtoCreateAndSerialize, "Protobuf from keywords and serialize."
    )

    self.TimeIt(
        ProtoCreateAndSerializeSetValue, "Protobuf Create, Set and serialize"
    )

    self.TimeIt(
        ProtoCreateAndSerializeFromProto,
        "Protobuf from serialized and serialize.",
    )

  def testObjectCreation2(self):

    def ProtoCreateAndSerialize():
      s = jobs_pb2.GrrMessage(
          name="foo", request_id=1, response_id=1, session_id="session"
      )
      return len(s.SerializeToString())

    def RDFStructCreateAndSerialize():
      s = StructGrrMessage(
          name="foo", request_id=1, response_id=1, session_id="session"
      )

      return len(s.SerializeToBytes())

    self.TimeIt(ProtoCreateAndSerialize, "Protobuf from keywords")

    self.TimeIt(RDFStructCreateAndSerialize, "RDFStruct from keywords")

  def testDecodeRepeatedFields(self):
    """Test decoding of repeated fields."""

    repeats = self.REPEATS // 50
    s = jobs_pb2.MessageList()
    for i in range(self.REPEATS):
      s.job.add(session_id="test", name="foobar", request_id=i)

    test_data = s.SerializeToString()

    def ProtoDecode():
      s = jobs_pb2.MessageList()
      s.ParseFromString(test_data)

      self.assertEqual(s.job[100].request_id, 100)

    def SProtoDecode():
      s = FastGrrMessageList.FromSerializedBytes(test_data)
      self.assertEqual(s.job[100].request_id, 100)

    self.TimeIt(SProtoDecode, "SProto Repeated Decode", repetitions=repeats)

    self.TimeIt(ProtoDecode, "Protobuf Repeated Decode", repetitions=repeats)

  def testRepeatedFields(self):
    """Test serialization and construction of repeated fields."""

    repeats = self.REPEATS // 50

    def ProtoCreateAndSerialize():
      s = jobs_pb2.MessageList()
      for i in range(self.REPEATS):
        s.job.add(session_id="test", name="foobar", request_id=i)

      return len(s.SerializeToString())

    def RDFStructCreateAndSerialize():
      s = FastGrrMessageList()

      for i in range(self.REPEATS):
        s.job.Append(session_id="test", name="foobar", request_id=i)

      return len(s.SerializeToBytes())

    self.TimeIt(
        RDFStructCreateAndSerialize,
        "RDFStruct Repeated Fields",
        repetitions=repeats,
    )

    self.TimeIt(
        ProtoCreateAndSerialize, "Protobuf Repeated Fields", repetitions=repeats
    )

    # Check that we can unserialize a protobuf encoded using the standard
    # library.
    s = jobs_pb2.MessageList()
    for i in range(self.REPEATS):
      s.job.add(session_id="test", name="foobar", request_id=i)

    serialized = s.SerializeToString()
    unserialized = FastGrrMessageList.FromSerializedBytes(serialized)
    self.assertLen(unserialized.job, len(s.job))

    self.assertEqual(unserialized.job[134].session_id, "test")
    self.assertEqual(unserialized.job[100].request_id, 100)

  def testDecode(self):
    """Test decoding performance."""

    s = jobs_pb2.GrrMessage(
        name="foo", request_id=1, response_id=1, session_id="session"
    )
    data = s.SerializeToString()

    def ProtoDecode():
      new_s = jobs_pb2.GrrMessage()
      new_s.ParseFromString(data)

      self.assertEqual(new_s.session_id, "session")
      self.assertIsInstance(new_s.session_id, str)

    def RDFStructDecode():
      new_s = StructGrrMessage.FromSerializedBytes(data)

      self.assertEqual(new_s.session_id, "session")
      self.assertIsInstance(new_s.session_id, str)

    self.TimeIt(RDFStructDecode)
    self.TimeIt(ProtoDecode)

  def testDecode2(self):
    """Test decoding performance.

    This benchmarks the lazy decoding feature where a large protobuf is decoded
    but only a few fields are examined.
    """

    s = knowledge_base_pb2.User(**self.USER_ACCOUNT)

    data = s.SerializeToString()

    def ProtoDecode():
      new_s = knowledge_base_pb2.User()
      new_s.ParseFromString(data)

      self.assertEqual(new_s.username, "user")
      self.assertIsInstance(new_s.username, str)

    def RDFStructDecode():
      new_s = rdf_client.User.FromSerializedBytes(data)

      self.assertEqual(new_s.username, "user")
      self.assertIsInstance(new_s.username, str)

    self.TimeIt(RDFStructDecode)
    self.TimeIt(ProtoDecode)

  def testEncode(self):
    """Comparing encoding speed of a typical protobuf."""
    s = jobs_pb2.GrrMessage(
        name="foo", request_id=1, response_id=1, session_id="session"
    )
    serialized = s.SerializeToString()

    def ProtoEncode():
      s1 = jobs_pb2.GrrMessage(
          name="foo", request_id=1, response_id=1, session_id="session"
      )

      test = s1.SerializeToString()
      self.assertLen(serialized, len(test))

    def RDFStructEncode():
      s2 = StructGrrMessage(
          name="foo", request_id=1, response_id=1, session_id="session"
      )

      test = s2.SerializeToBytes()
      self.assertLen(serialized, len(test))

    self.TimeIt(RDFStructEncode)
    self.TimeIt(ProtoEncode)

  def testEncodeDecode(self):
    """Test performance of encode/decode cycle."""

    def Check(s, new_s):
      self.assertEqual(s.name, new_s.name)
      self.assertEqual(s.name, "foo")
      self.assertEqual(s.request_id, new_s.request_id)
      self.assertEqual(s.request_id, 1)
      self.assertEqual(s.response_id, new_s.response_id)
      self.assertEqual(s.response_id, 1)
      self.assertEqual(s.session_id, new_s.session_id)
      self.assertEqual(s.session_id, "session")

    def ProtoEncodeDecode():
      s = jobs_pb2.GrrMessage(
          name="foo", request_id=1, response_id=1, session_id="session"
      )
      data = s.SerializeToString()

      new_s = jobs_pb2.GrrMessage()
      new_s.ParseFromString(data)

      return s, new_s

    def RDFStructEncodeDecode():
      s = StructGrrMessage(
          name="foo", request_id=1, response_id=1, session_id="session"
      )
      data = s.SerializeToBytes()

      new_s = StructGrrMessage.FromSerializedBytes(data)

      return s, new_s

    # Make sure everything is sane first.
    Check(*ProtoEncodeDecode())
    Check(*RDFStructEncodeDecode())

    self.TimeIt(RDFStructEncodeDecode)
    self.TimeIt(ProtoEncodeDecode)

  def testDecodeEncode(self):
    """Test performance of decode/encode cycle."""

    s = jobs_pb2.GrrMessage(
        name="foo", request_id=1, response_id=1, session_id="session"
    )
    data = s.SerializeToString()

    def ProtoDecodeEncode():
      new_s = jobs_pb2.GrrMessage()
      new_s.ParseFromString(data)
      new_s.SerializeToString()

    def RDFStructDecodeEncode():
      new_s = StructGrrMessage.FromSerializedBytes(data)
      new_s.SerializeToBytes()

    self.TimeIt(RDFStructDecodeEncode)
    self.TimeIt(ProtoDecodeEncode)


def main(argv):
  # Run the full test suite
  test_lib.main(argv)


if __name__ == "__main__":
  app.run(main)



#!/usr/bin/env python
import ipaddress

from absl.testing import absltest

from grr_response_core.lib.rdfvalues import client_network as rdf_client_network
from grr_response_proto import jobs_pb2


class NetworkAddressTest(absltest.TestCase):

  def testFromPackedBytesIPv4(self):
    ip = ipaddress.ip_address("192.168.0.1")

    address = rdf_client_network.NetworkAddress.FromPackedBytes(ip.packed)
    self.assertEqual(address.packed_bytes, ip.packed)
    self.assertEqual(address.address_type, jobs_pb2.NetworkAddress.INET)

  def testFromPackedBytesIPv6(self):
    ip = ipaddress.ip_address("2001:db8::8a2e:370:7334")

    address = rdf_client_network.NetworkAddress.FromPackedBytes(ip.packed)
    self.assertEqual(address.packed_bytes, ip.packed)
    self.assertEqual(address.address_type, jobs_pb2.NetworkAddress.INET6)

  def testFromPackedBytesIncorrect(self):
    with self.assertRaises(ValueError):
      rdf_client_network.NetworkAddress.FromPackedBytes(b"\xff\x00\xff")


if __name__ == "__main__":
  absltest.main()
